/**
 * Connection.java
 *
 * Copyright (C) 1997,1998,1999,2000,2001 Geometrek and 2009 by R. Russell Kinter
 *
 *  This file is part of DeepMatrixIP9.
 *
 *  DeepMatrixIP9 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrix IP9 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix;

import java.io.*;
import java.net.*;
/**
 * This class is the abstract basic super class for all types of connections.
 * Its implementations provide the semantics of a message stream over
 * various transportation layers.
 *
 * @see Message
 * @version 1.1
 */
public abstract class Connection {
/** flag storing whether connection is connected
 */
    protected boolean alive;
/** Default Constructor, clears alive flag
 */
    public Connection() {
        alive = false;
    }
/** opens a connection to the specified host and port. This does not indicate the
 * transportation layer. The given host:port could be a TCP stream, HTTP
 * tunneling or a reliable UDP connection, whatever.
 * @param host String giving the hostname
 * @param port port number to connect to
 * @throws IOException in case anything goes wrong
 */
    public abstract void open( String host, int port ) throws IOException, InterruptedException;
/** closes the connection
 */
    public abstract void close() throws InterruptedException;
 
    public abstract void sendMessage( Message msg ) throws IOException;

  /**
 * New 11-12-05 sendMessage2 is now handles Messages that have a priority
 * like update requests and changing room information. It circumvents the
 * two inner class MessageBuffers
 * sends a message over the connection
 * @param msg message to send
 * @throws IOException thrown if anything goes wrong
 */

   public abstract void sendMessage2( Message msg ) throws IOException, InterruptedException;

  /**
 * New 11-12-05 sendMessage3 handles ControlMessages that are chat text only
 * It adds them to TCPConnection's inner class MessageBuffer3
 * It was learned from extensive stress testing that large amounts of chat text
 * can kill the server.
 * @throws IOException thrown if anything goes wrong
 */

public abstract void sendMessage3( Message msg ) throws IOException;

/** reads a message from the connection
 * @throws IOException thrown if anything goes wrong
 * @return the last message read
 */

public abstract void exchange( ObjectInputStream in, ObjectOutputStream out ) throws Exception;

public abstract void startMsgBuffer();
    public abstract Message getMessage() throws IOException;
/** returns whether the connection is connected or not
 * @return boolean flag
 */

    public abstract boolean isAlive();
/** returns the InetAddress of the host the connection is currently connected to.
 * @return Address or null, if not connected
 */
    public abstract InetAddress getInetAddress();
/** returns the port the connection is connected to
 * @return the port or 0, if not connected
 */
    public abstract int getPort();

    public abstract void setSoTimeout( int iSTO ) throws SocketException;

    public abstract void setShutDown() throws IOException, InterruptedException;
}
