 /**
  ControlMessage.java

  Copyright (C) 1997,1998 Geometrekand 2009 by R. Russell Kinter
 
 *  This file is part of DeepMatrixIP9.
 *
 *  DeepMatrixIP9 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrix IP9 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package matrix;

import java.io.*;

/**
 * A control event message used between server and client
 * It defines the protocol used to lock shared events in a user's
 * name
 *
 * @version 1.1
 */
public class ControlEvent extends Message {
  final public static int CTRLEV     = 100,
                          CTRLCR	 = 200,
                          CTRLRM     = 300,
			              TYPE       = 9;
  public int command;
  public String name;
  public String [] arg;

  public ControlEvent() {}

  public ControlEvent( int command, String name, String [] arg ){
    this.command = command;
    this.name = name;
    this.arg = arg;
  }

  public ControlEvent( DataInputStream in ) throws IOException {
    read( in );
  }

  public int getType(){
    return TYPE;
  }

  public synchronized void read( DataInputStream in ) throws IOException {
    command = in.readInt();
    name = in.readUTF();
	arg = new String[in.readInt()];
    for (int i = 0; i < arg.length; i++) {
      arg[i] = in.readUTF();
    }
  }

  public synchronized void write( DataOutputStream out ) throws IOException {
    if( name == null || arg == null )
      throw new IOException("Data items are null");
    out.writeInt( command );
    out.writeUTF( name );
    out.writeInt(arg.length);
 for( int a = 0; a<arg.length; a++ ){
    out.writeUTF( arg[a] );
    }
  }

  public String toString(){
    return ""+command +" "+name+" "+arg[0]+" "+arg[1];
  }
}
