 /**
  ControlMessage.java

  Copyright (C) 1997,1998 Geometrekand 2009 by R. Russell Kinter
 
 *  This file is part of DeepMatrixIP9.
 *
 *  DeepMatrixIP9 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrix IP9 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package matrix;

import java.io.*;

/**
 * A control message used between server and client
 * It defines the protocol used and handles parsing and building
 * the messages.
 *
 * @version 1.1
 */
public class ControlMessage extends Message {
  final public static int LOGIN      = 100,
                          LOGOUT     = 150,
                          OBJADD     = 200,
                          OBJDEL     = 210,
			              OBJREQ     = 250,
                          ROOM       = 400,
                          NEWROOM    = 450,
                          UPDATE     = 600,
                          OBJUPD     = 650,
                          MSG        = 700,
		                  IGG        = 750,
                          WHOIS      = 800,
			              OBJIS      = 850,
			              ROOMIS     = 875,
			              HRTBT      = 900,
                          ADMIN      = 925,
                          ERROR      = 950,
			              TYPE       = 1;
  public int command;
  public String name,arg;

  public ControlMessage() {}

  public ControlMessage( int command, String name, String arg ){
    this.command = command;
    this.name = name;
    this.arg = arg;
  }

  public ControlMessage( DataInputStream in ) throws IOException {
    read( in );
  }

  public int getType(){
    return TYPE;
  }

  public synchronized void read( DataInputStream in ) throws IOException {
    command = in.readInt();
    name = in.readUTF();
    arg = in.readUTF();
  }

  public synchronized void write( DataOutputStream out ) throws IOException {
    if( name == null || arg == null )
      throw new IOException("Data items are null");
    out.writeInt( command );
    out.writeUTF( name );
    out.writeUTF( arg );
  }

  public String toString(){
    return ""+command +" "+name+" "+arg;
  }
}
