/*
  Backend.java

  Copyright (C) 1999 Geometrek	and 2009 by R. Russell Kinter
 
 *  This file is part of DeepMatrixIP9.
 *
 *  DeepMatrixIP9 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrix IP9 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */ 

package matrix.deck;
import matrix.server.HttpServletConnection;
import matrix.*;
import matrix.vrml.Field;
import java.io.*;
import java.net.*;
import java.awt.*;
import java.applet.*;
import java.util.*;

/**
 * Network Backend for Matrix client. helps to modularize
 * the client code. This class provides an interface to
 * to the network functionality and manages also the
 * RealTimeHandler. The client applet than only has to talk
 * to this class.
 *
 * @version 1.2
 */

public class Backend implements Network, Runnable {
    private Connection net = null;
    /**
     *  New 11-12-05 Indicates whether or not HttpClientConnection or TCPConnection has been implimented.
     *  If false then the MsgBuffer thread of TCPConnection is started via startMsgBuffer call.
     */
    private String sWarning = "foo";
    private String adminPassword = "none";
    private boolean hcc = false;
 // private RealTimeHandler rtThread;
    private RealTimeInfo rtThread;
    private Thread runner = null;
    private boolean bLogOut = false;
    private int port = 6666, clientID;
    private String name;
    private boolean bInit = false;
    private boolean connected = false;
    private Vector listeners;

    public Backend() throws IOException {
	//net = new Connection( 4096 );
	RealTimeHandler rth = new RealTimeHandler();
    rtThread = (RealTimeInfo)rth;
	listeners = new Vector();
    }

    public Backend( NetworkListener listener ) throws IOException {
    RealTimeHandler rth = new RealTimeHandler();
    rtThread = (RealTimeInfo)rth;
	listeners = new Vector();
	addNetworkListener( listener );
    }

    public Backend( NetworkListener listener, int type ) throws IOException {
    BotRTH brth = new BotRTH();
    rtThread = (RealTimeInfo)brth;
	listeners = new Vector();
	addNetworkListener( listener );
    }

    /**
     *  New 11-12-05 connect has new variable ARG for controlling when the new Room is loaded.
     * The ARG "A" tells the server to hold off delivering the signal to load the VRML.
     * control protocol methods. 
     * Network interface implementation.
     */

    public void connect( String hostname, int port, String name, String passwd, String room, String ARG )
	throws IOException {
  //rtThread.clear2();
    bInit = false;
    bLogOut = false;
  if( net != null ){
	if( net.isAlive())
	    throw new IOException(sWarning);
 }
 //"Allready connected to "+net.getInetAddress().getHostName()
	this.name = name;
    if( hostname.startsWith("http")) {
        hcc = true; net = new HttpClientConnection(); 
        } 
else {
 net = new TCPConnectionApplet( 4096 );
    }
 try {
	net.open(hostname, port);
  }
catch(InterruptedException g){}
  
ControlMessage msg = new ControlMessage( ControlMessage.MSG, "", "" );

	// wait for login prompt
	while( msg.command != msg.LOGIN ) {
	    msg = (ControlMessage) net.getMessage();
	}

	// send login
	msg.name = name;
	msg.arg = passwd;
	net.sendMessage( msg );
	// get connected !!!
	msg = (ControlMessage) net.getMessage();
    sWarning = msg.arg;
	if( msg.command != msg.MSG ) {
     System.out.println("Backend: Login failure: " +  msg.arg );
	   Enumeration list = listeners.elements();
	   while( list.hasMoreElements())
			    ((NetworkListener)list.nextElement()).handleNetFailure( msg.arg );  
        try{
        net.close();
         }
        catch (InterruptedException e){}
        net = null;
	    throw new IOException(msg.arg+" foo");
	}
	clientID = Integer.parseInt( msg.arg );
	if( room != null ){
	    msg.command = msg.ROOM;
	    msg.name = room;
	    msg.arg = ARG;
	    net.sendMessage( msg );
	}
	else{
	    msg.command = msg.MSG;
	    msg.name = name;
	    msg.arg = "hello";
	    net.sendMessage( msg );
	}
	runner = new Thread( this, "backend" );
	runner.start();
	connected = true;
    }

  public void startRTThread(){
  }

    /**
     * close connection to server and clears Consumers.
     */
    public void disconnect( String stf ){
	connected = false;
 if( net!=null ){
	if( net.isAlive() == true ) {
	    try {
		net.sendMessage( new ControlMessage( ControlMessage.LOGOUT, name, stf ));
	    }
	    catch(IOException e){ System.err.println("IOEx in disconnect : "+e);}
	} 
	else {
	    runner = null;
      }
     }
    }

    public boolean isConnected(){
	return connected;
    }

//Confirms VRML browser is Contact or Cortona in RealTimeHandler for update methods.
// a necessity for I.E. and Cosmoplayer combo to work.

  public void BackendupdateOk( boolean ok ){
  rtThread.rth_updateOk(ok); 
    }

    public void getStates() throws IOException {
	net.sendMessage( new ControlMessage( ControlMessage.UPDATE, "", "" ));
    }


 //New 4_05_04 whisper state info added to chat line.


    public void sendChatMessage( String to, String line ) throws IOException {
  String whisper; 
	if( to == null ){
	    to = "ALL"; whisper = line; }
 else { whisper = ("whispered to you : " + line.substring(11)); }
	net.sendMessage( new ControlMessage( ControlMessage.MSG, to, whisper ) );
    }

   /**
     *  New 11-12-05 sendHeartBeat polls server every 7 seconds.
     */

       public void sendHeartBeat( String name ) throws IOException {
    if(connected&&bInit){ net.sendMessage( new ControlMessage( ControlMessage.HRTBT, name, "PPPPPPP" )); }
  }

     /**
     *  New 11-12-05 Trying to coordinate the start of the MsgBuffer 
     *  of the inner Connection$MessageBuffer class before starting the Viewer thread.
     *  This will be done by RoomLoader calling gateFired2 method of Beta during the intitial loading. 
     */

    public void startMsgBuffer(){  
   }

 /**
  *
  *	07-05-09 All Ignore user issues will be handled by client.
  *
  */

	 public void sendIgnore( String target, String tf ) throws IOException {
 net.sendMessage( new ControlMessage( ControlMessage.IGG, target, tf ) );
 }

 public void askAdminControl( String name, String password ) throws IOException {
 net.sendMessage( new ControlMessage( ControlMessage.ADMIN, name, password ) );
 }

public void sendAdminControl( int icom, String nm, String com ) throws IOException {
 net.sendMessage( new AdminControl( icom, nm, com ) );
 }

    public void addSharedObject( String name, String url ) throws IOException {
	net.sendMessage( new ControlMessage( ControlMessage.OBJADD, name, url ));
    }

   public void sendRoomControl( RoomControl rc ) throws IOException {
   net.sendMessage( rc ); 
    }

   public void sendNewRoom( String name, String url ) throws IOException {
	net.sendMessage( new ControlMessage( ControlMessage.NEWROOM, name, url ));
    }
  


      public void deleteSharedObject( String name ) throws IOException {
	  net.sendMessage( new ControlMessage( ControlMessage.OBJDEL, name, "" ));
     }

	  public void sendControlEvent(  String name, String [] owner ) throws IOException {
     net.sendMessage( new ControlEvent( ControlEvent.CTRLEV, name, owner ));
     }

      public void sendControlRoom( String name, String [] owner ) throws IOException {
     net.sendMessage( new ControlEvent( ControlEvent.CTRLRM, name, owner ));
     }

   public void sendObjectRequest( String name, String state ) throws IOException {
     net.sendMessage( new ControlMessage( ControlMessage.OBJREQ, name, state ));
     } 


    public void update( String name, boolean obj ) throws IOException {
ControlMessage foo;
if(obj==false){
 foo = new ControlMessage( ControlMessage.UPDATE, name, "" );
}
else {
foo = new ControlMessage( ControlMessage.OBJUPD, name, "" );
 }
	net.sendMessage( foo );
    }

     /**
     *  New 11-12-05 switchRoom has been a new variable added. "arg" gives the
     *  server the command that the client is now ready to enter a new room
     */

        public void switchRoom( String name, String arg ) throws IOException {
	net.sendMessage( new ControlMessage( ControlMessage.ROOM, name, arg ));
    }

    public void whois() throws IOException {
	net.sendMessage( new ControlMessage( ControlMessage.WHOIS, "", "" ));
    }

    // adding / removing NetworkListener

    public void addNetworkListener( NetworkListener listener ){
	if( ! listeners.contains( listener ))
	    listeners.addElement( listener );
    }

    public void removeNetworkListener( NetworkListener listener ){
	listeners.removeElement( listener );
    }

    public void setNavChange( String [] types ){
	 Enumeration list = listeners.elements();
	  	while( list.hasMoreElements())
			    ((NetworkListener)list.nextElement()).handleNavChange( types );
    }

	public void setSpeedChange( float ft ){
	 Enumeration list = listeners.elements();
	  	while( list.hasMoreElements())
			    ((NetworkListener)list.nextElement()).handleSpeedChange( ft );
    }

    // real time protocol part

    public void addConsumer( RealTimeConsumer consumer ){
	rtThread.addConsumer( consumer );
    }

    public RealTimeConsumer getConsumer( String name ){
	return rtThread.getConsumer( name );
    }

    public void removeConsumer( String name ){
	rtThread.removeConsumer( name );
    }

    public Enumeration getConsumers(){
	return rtThread.getConsumers();
    }

    public void clearConsumers(){
	rtThread.clear();
    }

   //Delivers name request to RealTimeHandler for beamto drone Avatar.

   public void beamtotarget( String target ){
 rtThread.findbeamto( target );}

   public void setUserView( String target ){
 rtThread.setUserView( target );
 }

  public void setObjectRoutes( String target ){
 rtThread.setObjectRoutes( target );
 }



    public void sendRealTime( int stor, String tag, Field data, boolean echo ) throws IOException {
	net.sendMessage( new RealTimeMessage( stor, tag, data, echo ));
    }

	public void sendRealTimeSave( String tag, Field data, Field state ) throws IOException, InterruptedException{
	net.sendMessage( new RealTimeSave( tag, data, state ));
    }

    // end implementation of Network

    /**
     * Here is the main loop which talks to the Matrix server over TCP.
     * It notifies all Observers of all incoming messages ( also control
     * messages like LOGOUT and UPDATE ). The message is the argument to
     * the update method. <BR>
     * If an exception is thrown in the listening loop the connection is closed
     * and the exception is passed to the observer.
     */
    public void run() {
	Message msg;
	Enumeration list;
	try {
	    while( Thread.currentThread() == runner ) {
		try {
		    msg = net.getMessage();
		}
		catch(InterruptedIOException e){
		    continue;
		}
		switch( msg.getType()){
		case ControlMessage.TYPE :
		    ControlMessage cm = (ControlMessage)msg;
		    switch( cm.command ) {
		    case ControlMessage.UPDATE :
			rtThread.update( cm.name );
			break;
		    case ControlMessage.MSG :
			if(cm.name.equals(name))
			    continue;
			list = listeners.elements();
			while( list.hasMoreElements())
			    ((NetworkListener)list.nextElement()).handleMSG( cm.name, cm.arg );
			break;
		    case ControlMessage.OBJADD :
			list = listeners.elements();
			while( list.hasMoreElements())
			    ((NetworkListener)list.nextElement()).handleOBJADD( cm.name, cm.arg );
			break;
		    case ControlMessage.OBJDEL :
			list = listeners.elements();
			while( list.hasMoreElements())
			    ((NetworkListener)list.nextElement()).handleOBJDEL( cm.name );
            case ControlMessage.OBJREQ :
            break;
			case ControlMessage.IGG :
            break;
		    case ControlMessage.ERROR :
			list = listeners.elements();
			while( list.hasMoreElements())
			    ((NetworkListener)list.nextElement()).handleERROR( cm.name, cm.arg );
			break;
		    case ControlMessage.WHOIS :
			break;
			case ControlMessage.OBJIS :
			break;
            case ControlMessage.HRTBT :
            if(!bInit){ net.sendMessage( new ControlMessage( ControlMessage.HRTBT, name, "hrbt" ));  bInit = true; }
            break;
            case ControlMessage.NEWROOM :
			list = listeners.elements();
			while( list.hasMoreElements())
			    ((NetworkListener)list.nextElement()).handleNEWRM( cm.arg );
        	break;
		    case ControlMessage.ROOM :
			list = listeners.elements();
			while( list.hasMoreElements())
			    if(!cm.arg.equals("CB")){ 
     ((NetworkListener)list.nextElement()).handleROOM_B( cm.name, cm.arg ); }
     else { ((NetworkListener)list.nextElement()).handleROOM_A( cm.name );  }
			break;
		    case ControlMessage.LOGOUT :
    		bLogOut = true;
			list = listeners.elements();
	        while( list.hasMoreElements())
		   ((NetworkListener)list.nextElement()).handleLOGOUT();
            System.out.println("Logout from server");
			runner = null;
			break;
		    default :
			System.err.println("unrecognized/not handled Message "+cm);
		    }
		    break;
		case RealTimeMessage.TYPE :
		    rtThread.handle((RealTimeMessage)msg);
		    break;
		case WhoWhatWhere.TYPE :
		    list = listeners.elements();
			while( list.hasMoreElements())
			    ((NetworkListener)list.nextElement()).handleWWW( (WhoWhatWhere)msg );
		    break;
        case ControlEvent.TYPE :
       ControlEvent cv = (ControlEvent)msg;
		switch( cv.command ){
			case ControlEvent.CTRLEV :
			list = listeners.elements();
			while( list.hasMoreElements())
			((NetworkListener)list.nextElement()).handleCTRLEV( cv.name, cv.arg );
			break;
    		case ControlEvent.CTRLCR :
            break;
            case ControlEvent.CTRLRM :
            break;
			default :
		    System.err.println("unrecognized/not handled Message "+cv.toString());
           }
           break;
		case AdminControl.TYPE :
           AdminControl ac = (AdminControl)msg;
			switch( ac.command ){
                 case  AdminControl.HELLO :
                   adminPassword = ac.arg;
				   list = listeners.elements();
			while( list.hasMoreElements())
			    ((NetworkListener)list.nextElement()).handleADMIN( 1 );
                    break;
                 case  AdminControl.DONE :
				    list = listeners.elements();
			while( list.hasMoreElements())
			    ((NetworkListener)list.nextElement()).handleADMIN( 2 );
                    break;
                 case  AdminControl.NEWOBJ : 
					break;
                 case  AdminControl.REMOBJ :
					break;
                 case  AdminControl.NEWUSR :
					break;
			     case  AdminControl.BANUSR :
					break;
				 case  AdminControl.BANADD :
					break;
                 case  AdminControl.APRVRM :
					break;
                 case  AdminControl.REJCT :
				 list = listeners.elements();
			while( list.hasMoreElements())
			    ((NetworkListener)list.nextElement()).handleMSG( "MatrixD", ac.arg );
                    break; 
				 default :
				  System.err.println("unrecognized/not handled Message "+ac.toString());
                break;
                  }
			case RealTimeSave.TYPE :
            break;
			case RoomControl.TYPE :
			list = listeners.elements();
			while( list.hasMoreElements())
			    ((NetworkListener)list.nextElement()).handleWWW( msg );
		    break;
		default :
		    System.err.println("unknown message type "+msg);
		}
	    }
	}
 catch(InterruptedException f){ System.err.println("Backend run IntrrptExp: "+f);}
	catch(IOException e) { 
   System.err.println("IOException Backend: " + e);
	    System.err.println("error listening in main loop\n\t"+e);
	    disconnect("false");
	    // simulate a logout message here or introduce error callback in nwlistener
	    list = listeners.elements();
	    while( list.hasMoreElements())
		((NetworkListener)list.nextElement()).handleLOGOUT();
	}
  catch(Exception g){
   System.err.println("Backend :Logout from Exception:\n\t"+g);
	    disconnect("false");
	    // simulate a logout message here or introduce error callback in nwlistener
	    list = listeners.elements();
	    while( list.hasMoreElements())
		((NetworkListener)list.nextElement()).handleLOGOUT();
     }
	finally {
   try {  
     net.close();
     }
catch(InterruptedException g){}
	}
    }

	public void handleUserPosition( float [] flt ){
     rtThread.setUserPosition( flt );
    }

    public void setViewer( Viewer viewer ){
        if( rtThread != null ){
            rtThread.setViewer( viewer );
         }
    }
    
    public void close(){
    runner = null;
    rtThread.close();
    }
}
