/*
  HttpClientConnection.java
 
  Copyright (C) 2001 Geometrek
 
 *  This file is part of DeepMatrixIP9.
 *
 *  DeepMatrixIP9 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrix IP9 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.deck;

import matrix.*;
import java.io.*;
import java.net.*;
import java.util.*;
/**
 *
 * @author  gr
 */
public class HttpClientConnection extends Connection implements Runnable {
    
    // the servlet url to connect to
    protected URL url;
    
    // session cookie send by the servlet
    protected String sessionCookie = null;
    protected String name = null;
    
    // inetaddress of the web server running the servlet
    protected InetAddress address;
    
    /** list of incoming and outgoing messages
     */
    protected Vector input = new Vector(), output = new Vector();
    
    /** thread driving the connection */
    Thread runner;
        
    /** Creates new HttpClientConnection.
     * @param address url to connect to
     */
    public HttpClientConnection( String address ) throws IOException {
        open( address, 80 );
    }
    
    /** Creates new HttpClientConnection. */
    public HttpClientConnection(){
    }
    
    /** opens a connection to the specified host and port. This does not indicate the
     * transportation layer. The given host:port could be a TCP stream, HTTP
     * tunneling or a reliable UDP connection, whatever.
     * @param host url to connect to
     * @param port is ignored at it is expected to be part of the url
     * @throws IOException in case anything goes wrong
     */
    public void open( String host, int port ) throws IOException {
        close();
        url = new URL( host );
        address = InetAddress.getByName( url.getHost());
        runner = new Thread( this, "HttpClientConnection");
        runner.start();
        alive = true;
    }
    
    /** closes the connection
     */
    public void close(){
        if( isAlive()){
            // maybe send some internal protocol command to signal a closed
            // connection.
        }
        alive = false;
    }

    /** sends a message over the connection
     * @param msg message to send
     * @throws IOException thrown if anything goes wrong
     */
    public void sendMessage( Message msg ) throws IOException{
        if( !isAlive()) {
            throw new IOException("Connection not connected !");
        }
        output.addElement( msg.copy() );
    }
    
     public void sendMessage2( Message msg ) throws IOException{
        if( !isAlive()) {
            throw new IOException("Connection not connected !");
        }
        output.addElement( msg.copy() );
    }

       public void sendMessage3( Message msg ) throws IOException{
        if( !isAlive()) {
            throw new IOException("Connection not connected !");
        }
        output.addElement( msg.copy() );
    }

    /** reads a message from the connection
     * @throws IOException thrown if anything goes wrong
     * @return the last message read
     */
    public Message getMessage() throws IOException {
        if( !isAlive()) {
            throw new IOException("Connection not connected !");
        }
        Message msg;
        try {
            synchronized( input ){
                if( input.size() == 0 ){
                    input.wait();
                }
            msg = (Message) input.firstElement();
            input.removeElementAt(0);
          }
        }
        catch( InterruptedException e ){
            throw new InterruptedIOException("Timeout");
        }
        return msg;
    }
 
  public  void exchange( ObjectInputStream in, ObjectOutputStream out ) throws Exception {
  }

     public void startMsgBuffer(){
     }

	  public void setSoTimeout( int iSTO ) throws SocketException {
     } 

    /** returns the InetAddress of the host the connection is currently connected to.
     * @return Address or null, if not connected
     */
	public boolean isAlive() { return alive; }

    public InetAddress getInetAddress() {
        if( isAlive() ){
            return address;
        }
        return null;
    }
    
    /** returns the port the connection is connected to
     * @return the port or 0, if not connected
     */
    public int getPort(){
        if( isAlive()){
            return url.getPort();
        }
        return 0;
    }
    
    /** connects to the servlet at regular intervalls and exchanges arrays of
     * messages */

    public void run() {
        Message[] msgs;
        int errorCount = 0;
        while( Thread.currentThread() == runner ){
            try {
                // send a request
                URLConnection con = url.openConnection();
                con.setUseCaches(false);
                con.setDoOutput(true);
                con.setDoInput(true);
                if( sessionCookie != null ){
                    con.setRequestProperty("Cookie",sessionCookie);
                }
                
                ByteArrayOutputStream byteOut = new ByteArrayOutputStream();
                ObjectOutputStream out = new ObjectOutputStream(byteOut);
                
                synchronized( output ){
                    msgs = new Message[ output.size() ];
                    output.copyInto( msgs );
                    output.removeAllElements();
                }
                out.writeObject( msgs );
                out.flush();
                byte buf[] = byteOut.toByteArray(); // Convert to the byte array
                // Set the Request Property Content-type and Content-length
                con.setRequestProperty("Content-type","application/octet-stream");
                con.setRequestProperty("Content-length"," " + buf.length);
                OutputStream dataOut = con.getOutputStream();
                dataOut.write(buf);
                dataOut.flush();
                dataOut.close();
                
                // Read the Response Object Input Stream
                ObjectInputStream in = new ObjectInputStream( con.getInputStream ());
                msgs = (Message [])in.readObject();
                in.close ();
                for( int i = 0; i < msgs.length; i++ ){
                    input.addElement( msgs[i] );
                }
                if( msgs.length > 0 ){
                    synchronized( input ){
                        input.notifyAll();
                    }
                }
                String cookie = con.getHeaderField("Set-Cookie");
                if( cookie != null ){
                    cookie = cookie.substring(0,cookie.indexOf(';'));
                    if( !cookie.equals( sessionCookie )){
                        sessionCookie = cookie;
                        System.out.println("Got a new cookie "+sessionCookie);
                    }
                }
                errorCount = 0;
            }
            catch(Exception e){
                System.err.println("Error tunneling "+e);
                errorCount++;
            }
            msgs = null;
            if( errorCount > 3 ){
                close();
            }
            if( !isAlive() ){
                sessionCookie = null;
                runner = null;
                input.removeAllElements();
                output.removeAllElements();             
                continue;
            }
            try {
                Thread.sleep( 1000 );
            }
            catch(Exception e){}
        }
    }

   public void setShutDown() throws IOException, InterruptedException {
  }

}