/*
  NetworkListener.java

  Copyright (C) 1998 Geometrek	and 2009 by R. Russell Kinter
 
 *  This file is part of DeepMatrixIP9.
 *
 *  DeepMatrixIP9 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrix IP9 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 

package matrix.deck;

import matrix.Message;

/**
 * The NetworkListener interface is implemented by classes interested in network messages.
 *
 * @version 1.1
 */
public interface NetworkListener {


/**
 * called if a a LOGOUT message is received. A LOGOUT message is received after disconnect of
 * the Network interface is called and the connection is closed. Also generated if Network closes
 * after an error is encountered.
 */
  public void handleLOGOUT();

/**
 * called if a new object is added. Only for objects added from other clients. The Network
 * class does not add a consumer object for a new object. If this is desired the application
 * has to provide that itself.
 * @param name name tag of the object
 * @param url the URL of the VRML file representing the object
 */
  public void handleOBJADD( String name, String url );

/**
 * called if an object is deleted. Again only for objects managed by other clients. also
 * no consumers are deleted. Again left to the application.
 * @param name name of the deleted object.
 */
  public void handleOBJDEL( String name );

  public void handleCTRLEV( String target, String [] lock );

  public void  handleOBJIGN( String name, String tf );
/**
 * New 11-12-05 Methods handleRoom_A and B are for intially loading the entry room or loading a new room.
 * switchRoom() of the Network sends first notice of desired room to the server. 
 * The server keeps the user in stasis for a four seconds so the user is not in any particular room
 * so can not receive any messages. During this time the client goes through memory clearing. 
 * Then the RoomLoader class sends another message to the server which in turn calls handleROOM_B
 * that now sends the room name and url to the browser to load.
 * Originally it was just one method "handleRoom" But the four second respite adds robustness to the client.
 * @param name name of the room
 * @param url URL of the VRML file representing the room
 */
  public void handleROOM_A( String name ) throws InterruptedException;


  public void handleROOM_B( String name, String url ) throws InterruptedException;

  public void handleNetFailure( String reason );


/**
 * called if a message ( chat ) is received.
 * @param from the senders name
 * @param line the message line
 */
  public void handleMSG( String from, String line );

/**
 * called if an update to the user list is received.
 * @param users an array holding the current users names
 */
  public void handleWHOIS( String[] users );
  
  public void handleOBJIS( String[] objects );

  public void handleWWW( Message msg );

  public void handleNEWRM( String new_room );

  public void handleADMIN( int arg );

/**
 * called if an error message is received. typically error messages are generated on wrong
 * syntactically or semantically messages send to the server. For example if you want to switch
 * to a room that does not exist. Also called if the Network class itself encounters an error.
 * @param from the originator of the error message ( typically the DM system )
 * @param line the error message
 */
  public void handleERROR( String from, String line );

 // Local calls making use of interface.

  public void handleNavChange( String [] types );

  public void handleSpeedChange( float ft );


  //public String getUserName();

}
