/**
  RealTimeConsumer.java

  Copyright (C) 1998 Geometrek	and 2009 by R. Russell Kinter
 
 *  This file is part of DeepMatrixIP9.
 *
 *  DeepMatrixIP9 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrix IP9 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.deck;

import matrix.vrml.Field;



/**
 * Interface for any classes consuming and handling RealTimeMessages
 *
 * @version 1.0
 * @see matrix.deck.eai.Avatar
 * @see matrix.deck.eai.Route
 * @see custom_nodes.NetworkState
 */
public interface RealTimeConsumer {


/**
 * must be unique on client to identify the consumer
 */
  public String getName();

  public boolean getbAvatar();
/**
 * handles an incoming message from the network.
 * @param tag the tag name of the update
 * @param data the new data
 */
  public void handleMessage( String tag, Field data ) throws ConsumerException;

/**
 * Consumer should send its last state out no matter what. Used for updating
 * clients that connect at a later time.
 */
  public void update();

/**
 * notifies a Consumer that it will be deleted. The consumer should clear any references
 * to VRML objects it still holds.
 */
  public void delete();
 /**
 * notifies a Consumer that it will be beamed. The consumer should pass position and orientation
 * to UserAvatar.
 */
  public void beamto( String target );

   public void userView( String target );

  public void setUserPosition( float[] flt );
  
  public void objectRoutes( String target );

  public void checkPosition( float[] fUP, boolean bUPs  );
  
}
