/*
  UserInterface.java

  Copyright (C) 1998 Geometrek and 2009 in part by Russ Kinter
 
 *  This file is part of DeepMatrixIP9.
 *
 *  DeepMatrixIP9 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrix IP9 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */ 

package matrix.deck;

import matrix.Message;

/**
 * defines the user interface functionality. This basic interface allows only to set the data
 * for the UI. The UIs functionality is
 * <ul>
 * <li> returning users login name, password and avatar URL upon connecting </li>
 * <li> sending chat lines along with selected users name </li>
 * <li> events for gestures with gesture name selected </li>
 * <li> events for ghost mode action ( with true / false ) </li>
 * <li> events for disconnect </li>
 * </ul>
 *
 * @see matrix.deck.UserInterfaceListener
 * @version 1.1
 */

public interface UserInterface {

// how to set initial data ?
// if a class is loaded at runtime, we only have the generic constructor ! ( in JDK 1.0 )

/**
 * switches gui to login panel, clears fields and sets the current value of any flag
 * keeping track of ghost mode to not in ghost mode.
 */
  public void showLoginView();

/**
 * switches gui to run panel and clears fields too
 */
  public void showRunView();

/**
 * sets the avatar list in the login part list box
 *
 * @param names list of avatar names
 * @param urls  list of urls corresponding to avatar name, same size as names
 */
  public void setAvatars( String[] names, String[] urls );

/**
 * sets gesture names in run part
 *
 * @param names list of gesture names
 */
  public void setGestures( String[] names );

/**
 * sets the current users list in run part
 */
  //public void setUsers( String[] names );
  
  //public void setObjNames( String[] objnames );

  public void setWWW( Message msg );

/**
 * sets status in login part or general ( depends on implementation )
 */
  public void setStatus( String line );

/**
 * sets Ghost mode status
 *
 * @param value flag whether Ghost mode is set or not
 */
  public void setGhostMode( boolean value );

/**
 * sets Beamto mode status
 *
 * @param value flag whether Beamto mode is set or not
 */
  //public void setBeamto( String value );

/**
 * displays a line in the textArea
 */
  public void display( String from, String line );

  public void setNavChange( String [] types );

  public void setSpeedChange( float ft );


  public void setLableAnchorInfo( boolean bBC, String mst [] );


  public  void setAdmin( int arg );

/**
 * adds a Listener
 */
  public void addUserInterfaceListener( UserInterfaceListener listener );

/**
 * removes a Listener
 */
  public void removeUserInterfaceListener( UserInterfaceListener listener );
}
