/**
  Avatar.java

  Copyright (C) 1998 Geometrek and 2005 in part by Russ Kinter

 *  This file is part of SET Matrix 1.
 *
 *  SET Matrix 1 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  SET Matrix 1 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package matrix.deck.eai;

import matrix.vrml.*;
import matrix.deck.RealTimeConsumer;
import matrix.deck.ConsumerException;
import matrix.deck.Network;
import matrix.deck.NetworkListener;
import java.io.*;
import java.util.*;
import vrml.external.*; 
import vrml.external.field.*;
import vrml.external.exception.*;


/**
 *  manages a single avatar
 *
 *  @version 1.3
 *
 * New 11-12-05 the gesture list is now also used to list Avatar options 
 * (NG_chat_text,NG_humanoid,NG_no_lod) "NG_" prefix is for the UserAvatar
 * so the option does not show up as a gesture. 
 * NG_chat_text means the avatar is setup for chat text display.
 * NG_humanoid the avatar has Deep Matrix's humanoid animation methods: "inMotionBehaviour" 
 * and "notInMotionBehaviour"
 * NG_no_lod means the avatar does not want the built in LOD
 *
 * drone_av_finished( false ) is called from Beta if the avatar is either finished or throws an error.
 */
public class Avatar implements RealTimeConsumer {
    /**
    * New 11-12-05 AvatarTextTalk class for delivering chat text strings to drone avatar display.
    */
    private AvatarTextTalk texttalk = null;
    /**
    * New 11-12-05 String variables for keep track of avatar features
    */
    private String talk = "nah",human = "hum", lod = "bah", inMotionBehaviour = null, notInMotionBehaviour = null; 
     /**
    * New 11-12-05 Lodst is for built-in drone Avatar Level of Detail to be created to VRML from String.
    */
    private String Lodst = ( "Group { " +   
             "children [ " + 
             " LOD { " + 
             "range [40 80] " +   
             " level [ " +  
             "Transform { " +
						       "rotation 0 1 0 3.141 " +
                               "addChildren IS addLOD_Children " +
            				   "removeChildren IS removeLOD_Children " +
						       "children IS lod_children " +  
             "  }  " +   
             "Transform {  " +  
             "translation	0 0 0 " + 
             "children [	Shape {  " +
             "appearance	NULL  " + 
             "geometry	 IndexedFaceSet {  " + 
             "coord	Coordinate {  " + 
	            " point	[ -0.25 1.75 0.125, "  +
		  " -0.25 0.0 0.125,"  +
		  " 0.25 1.75 0.125, "  +
		  " 0.25 0.0 0.125, "  +
		  " 0.25 1.75 -0.125, "  +
		  " 0.25 0.0 -0.125, "  +
		  " -0.25 1.75 -0.125, "  +
		  " -0.25 0.0 -0.125 ] "  +
     "  } "  +
     " coordIndex	[ 0, 1, 3, 2, -1, 4, 5, 7, "  +
	       "  6, -1, 6, 7, 1, 0, -1, 2, "  +
	       "  3, 5, 4, -1, 6, 0, 2, 4, "  +
	       "  -1, 1, 7, 5, 3, -1 ] "  +
             "color Color { color [ .3 .3 .3, .3 .3 .3, .3 .3 .3, .3 .3 .3, .3 .3 .3, .3 .3 .3, .3 .3 .3,  .3 .3 .3 ]}  " +  
             "}}]}  " + 
             "Group { } " + 
             "]}]} " );

	/**
    * New 11-12-05 field2 and field2 hold the talker_text and set_name eventIns of a drone avatar if found.
    */
    private EventInSFString set_name = null;
   /**
   /**
    * New 11-12-05 field2 and field2 hold the talker_text and set_name eventIns of a drone avatar if found.
    */
    private EventIn field2, field3;
   /**
    * New 11-12-05 field holds Avatar gestures MFString
    */
    private EventOut field;
   /**
    * New 11-12-05 Beta is added as variable for beamto
    */
    private Beta avdeck;
     /**
    * New 11-12-05 a is used as a boolean to determine if chat text is to be displayed.
    */
    private int a = 0;
     /**
    * New 11-12-05 collects position and rotation together for beam to information
    */ 
    private float [] beam = new float[7];
    private float [] fRange = new float [0];
   /**
    * New 11-12-05 position with beam to information
    */ 
    private float [] posflt = new float[3];
   /**
    * New 11-12-05 rotation with beam to information
    */ 
    private float [] rotflt = new float[4];
    private String name;
    private Network postOffice;
	private EventInSFInt32 NameState = null;
     /**
    * New 11-12-05 Node for Chat
    */
    private EventInMFString yak = null;
     /**
    *  Holds the avatar
    */
    private Node avatar = null; 
   /**
    *  Grouping Node
    */
    private Node anchor = null;

   /**
    *  holds avatars position.
    */
    private EventInSFVec3f pos;


	private EventInSFInt32 which;
    private EventInSFRotation rot;
    /**
    * New 11-12-05 pos2 EventOuts for beam to coordinate position.
    * Same exposedField as "pos" 
    */
    private EventOutSFVec3f pos2 = null;
   /**
    * New 11-12-05 rot2 EventOuts for beam to coordinate rotation
    * Same exposedField as "rot"  
    */
    private EventOutSFRotation rot2 = null;
    /**
     * New 11-12-05 name_text is for users name display to be create VRML from String
     */
    private String name_text; 
    private String avUrl = "";
    
    private String name2 = "";
 
	public boolean bAvatar = false;
   /**
    * holds Routes for found Shared Event Protos
    */
    private Vector states = new Vector();
    public boolean bAvOk = true, bObject = false, bname_state = true, bOR = true, bP = true, bO = true;

    /**
     * builds a new Avatar. It parses the loaded nodes and construct the needed scenegraph
     * and gets the events etc. Also builds all the necessary Route objects, if it is
     * a SharedObject.
     *
     * @param object the loaded object, containing the name and a Transform
     * @param browser reference to the VRML browser
     * @param postOffice reference to the Network object
     *
     */

    public Avatar( SO1 object, Browser browser, Network postOffice, Beta deck, int VIN, long slp ) throws RuntimeException {
  Node [] Foo = null;
 Node [] xform = null;
 Node [] nodes = null;
 Node [] help3 = null;
 name = object.getName();
 avUrl = object.getUrl();
System.out.println("Avatar name: " + name );
System.out.println("Avatar url: " + avUrl );
 if(name.endsWith("_Obj")){
 name2 = name.substring(1,name.length()-4);
 }
 else {
 name2 = name.substring(1);
 }
	this.postOffice = postOffice;
 avdeck = deck;
 name_text = ( "Transform { " + 
						       "translation 0 3.75 0 " +
						       "children [ " + 
             "LOD {  " +
             "range 20 " +                                         
             "level [ " +
						       "Billboard {   " +
						       "children [  " +
						       "Shape { geometry Text { string [\"" + name.substring(1) + "\"] maxExtent 4  " +
						       "fontStyle FontStyle { family [\"SANS\"] justify \"MIDDLE\" size .5 } } " +
						       "} " +
             "]} " +
             "Group {} " +                                          
						       "]}]}  " );
	// get nodes
		try {
 xform = object.getTransforms2();
    }
  catch(NullPointerException np){ System.out.println( "object.getTransforms(): " + np );
   avdeck.drone_av_finished( false );
	    return;}
if(xform[0].getType().equals("Avatar3")){
  	try {
 Foo = object.getTransforms1();
 nodes = ((EventOutMFNode)Foo[0].getEventOut("children_changed")).getValue();
    }
  catch(NullPointerException np){ System.out.println( "object.getTransform(): " + np );
  avdeck.drone_av_finished( false );
	    return; 
     }
	// check for SharedObject definition
	if( nodes[0].getType().equals("SharedObject")){
	    Node soNode = nodes[0];
 EventOut foo = null;
 try {
foo = soNode.getEventOut("states_changed");
 }
catch(NullPointerException e){ 
System.out.println( "Avatar SharedObject no states_changed: " + e);
}
catch(InvalidEventOutException f){ 
System.out.println( "Avatar SharedObject no states_changed: " + f);
}
catch(InvalidVrmlException g){ 
System.out.println( "Avatar SharedObject no states_changed: " + g);
}
catch(Exception h){
System.out.println( "Avatar SharedObject no states_changed: " + h);
} 
try {
if(foo!=null&&foo.getType()==FieldTypes.MFNODE){ 
	    Node[] statesVal = ((EventOutMFNode)foo).getValue();
	    if( statesVal.length != 0 ){
		for( int i = 0; i < statesVal.length; i += 1 ){
			if(!deck.bLogOut){
		    if( statesVal[i].getType().startsWith("Network")){
			Route route = new Route( statesVal[i], postOffice, VIN, slp );
			route.setName( name+"_"+route.getName());
			postOffice.addConsumer( route );
			states.addElement( route );
		     }
            }
	      }
		}
      }
   }
catch(NullPointerException e){ 
System.out.println( "Avatar SharedObject states_changed error: " + e);
}
catch(Exception h){
System.out.println( "Avatar SharedObject states_changed errp: " + h);
}   
	  help3 = browser.createVrmlFromString("Transform { "+
						       "children [ "+
						       "Transform { "+
						       "translation 0 3 0 "+
						       "children [ "+
						       "Billboard { "+
						       "children [ "+
						       "Shape { geometry Text { string [\"" + name2 + "\"] maxExtent 4"+
						       "fontStyle FontStyle { family [\"SANS\"] justify \"MIDDLE\" } }"+
						       "}"+
						       "]}"+
						       "]}"+
						       "]}");
	    ((EventInMFNode)xform[0].getEventIn("addTXTChildren")).setValue(help3);
	    
	    // get events
	    pos = (EventInSFVec3f)xform[0].getEventIn("translation");
	    rot = (EventInSFRotation)xform[0].getEventIn("rotation");
		bObject = true;
     } 	
if( nodes[0].getType().equals("Avatar")){
try {
   field = nodes[0].getEventOut("gestures");
	    // workaround for Blaxxun Contact Beta bug
	    // getEventOut should throw Exception not return null
	    if( field == null )
		throw new InvalidEventOutException();
	    if( field.getType() != FieldTypes.MFSTRING)
		throw new InvalidEventOutException();
   int b = (((EventOutMFString)field).getValue()).length;
   String all [] = new String[b];
   all = (((EventOutMFString)field).getValue());
     for (int c = 0; c<=b; c++){ if(c<b){
if( all[c].equals("NG_chat_text")){talk = all[c];}
if( all[c].equals("NG_humanoid")){human = all[c]; }
if( all[c].equals("NG_no_lod")){lod = all[c]; }}
 if(c==b){
	    avatar = nodes[0];}}
	}
	    // workaround for Blaxxun Contact Beta bug
	    // getEventOut should throw Exception not return null
catch(InvalidEventOutException e){
	   a = -1;avatar = null;
    bAvOk = false;
    System.out.println(name+" is not valid VRML");
    avdeck.drone_av_finished( false );
	}
	// this bit is for Cortona which throws a NullPointer          
	// instead of an InvalidEventOut  !
	catch(NullPointerException e){
	   a = -1;avatar = null;
    bAvOk = false;
    System.out.println(name+" is not valid VRML");
    avdeck.drone_av_finished( false );
	}
if(avatar!=null&&talk.equals("NG_chat_text")){
try { 
 yak = (EventInMFString)nodes[0].getEventIn("talker_text");
  }
		catch(InvalidEventInException e){
	   a = -1;avatar = null;
    bAvOk = false;
    System.out.println(name + ": eventIn talker_text is either non existent or incorrect code! ");
    avdeck.drone_av_finished( false );
    return;
	}
	// this bit is for Cortona which throws a NullPointer          
	// instead of an InvalidEventIn  !
	catch(NullPointerException e){
	a = -1;avatar = null;
    bAvOk = false;
    System.out.println(name + ": eventIn talker_text is either non existent or incorrect code! ");
    avdeck.drone_av_finished( false );
    return;
	}
   try { 
   set_name = (EventInSFString)nodes[0].getEventIn("set_name");
   a = 1;
   }
		catch(InvalidEventInException e){
    a = -1;avatar = null;
    bAvOk = false;
    System.out.println(name + ": eventIn set_name is either non existent or incorrect code! ");
    avdeck.drone_av_finished( false );
    return;
	}
	// this bit is for Cortona which throws a NullPointer          
	// instead of an InvalidEventIn  !
	catch(NullPointerException e){
     a = -1;avatar = null;
     bAvOk = false;
     System.out.println(name + ":  eventIn set_name is either non existent or incorrect code! ");
     avdeck.drone_av_finished( false );
     return;
	}
 }
if(avatar!=null&&human.equals("NG_humanoid")){ 
	    try {
		inMotionBehaviour = ((EventOutSFString)(nodes[0].getEventOut("inMotionBehaviour"))).getValue();
		notInMotionBehaviour = ((EventOutSFString)(nodes[0].getEventOut("notInMotionBehaviour"))).getValue();
	    }
	catch(InvalidEventOutException e){
   		inMotionBehaviour = null;
		notInMotionBehaviour = null;
  avatar = null;a = -1; bAvOk = false;
	System.out.println("avatar does not define motion behaviours" );
  avdeck.drone_av_finished( false );
    return;
	}
	// this bit is for Cortona which throws a NullPointer          
	// instead of an InvalidEventIn  !
	catch(NullPointerException e){
   		inMotionBehaviour = null;
		notInMotionBehaviour = null;
  avatar = null;a = -1; bAvOk = false;
	System.out.println("avatar does not define motion behaviours" );
  avdeck.drone_av_finished( false );
     return;
	}
	    catch (Exception ex) {
		inMotionBehaviour = null;
		notInMotionBehaviour = null;
  avatar = null;a = -1; bAvOk = false;
	System.out.println("avatar does not define motion behaviours" );
  avdeck.drone_av_finished( false );
  return;
	    }	
    }
 }
if(bAvOk==true){
	    // get events
  pos = (EventInSFVec3f)xform[0].getEventIn("translation");
	 rot = (EventInSFRotation)xform[0].getEventIn("rotation");
  pos2 = (EventOutSFVec3f)xform[0].getEventOut("translation");
	 rot2 = (EventOutSFRotation)xform[0].getEventOut("rotation");
  which = (EventInSFInt32)xform[0].getEventIn("whichChoice");
 if(lod.equals("NG_no_lod")){
   ((EventInMFFloat)xform[0].getEventIn("range")).setValue(fRange);
  }
 anchor = xform[0];
 /**
     * New 11-12-05 If the avatar wants the chat display option this 
     * creates the AvatarTextTalk class to implement the option.
     */

if(a==1){  
   texttalk = new AvatarTextTalk( postOffice );
    set_name.setValue(name2);
   texttalk.set_yakker(yak); 
	System.out.println("Drone is text talker. ");
    }
  else {
 try {
  help3 = browser.createVrmlFromString( name_text );
 ((EventInMFNode)xform[0].getEventIn("addTXTChildren")).setValue(help3);
  }
 catch (InvalidVrmlException g) { System.out.println("Avatar drone Invalid VRML prob with name_text: " + g );
 avdeck.drone_av_finished( false );
  return; } 
catch (InvalidNodeException d) { System.out.println("Avatar drone Node prob with name_text: " + d );  
 avdeck.drone_av_finished( false );
  return; }
catch(NullPointerException f){System.out.println("Avatar drone Null prob with name_text: " + f );
 avdeck.drone_av_finished( false );
  return; } 
  }
  if(bObject==true){ avdeck.handleObjNames( name, avUrl ) ;}
  avdeck.drone_av_finished( true );
       }
     }
 else { 
avdeck.drone_av_finished( false );
return; 
  }
} 
  
 public boolean getbObject(){
  return bObject;
 }

 public String getName(){
	return name;
    }

    /**
     * handles incoming real time messages addressed to this avatar.
     * it understands three different types :
     * <ul><li> SFVEC3F for position updates</li>
     *     <li> SFROT for orientation updates</li>
     *     <li> SFSTRING to trigger a behaviour</li></ul>
     * @param tag tag of the incoming message
     * @param data the field encoded in the message.
     */
    public void handleMessage( String tag, Field data ) throws ConsumerException {
   float [] pflt = new float[3];
   float [] rflt = new float[4];
	if(!tag.equals(name))
	    throw new ConsumerException(name+" is not equal to "+tag);
	try {  
	    switch(data.getType()){
	    case Field.SFVEC3F:
		pos.setValue(((SFVec3f)data).getValue());
        pflt = ((SFVec3f)data).getValue(); 
       //System.out.println("Av " + name + " handleMessage SFVec3f: " + pflt[0] + " " + pflt[1] + " " + pflt[2]  );
		break;
	    case Field.SFROTATION:
        rflt = ((SFRotation)data).getValue();
		rot.setValue(((SFRotation)data).getValue());
		//System.out.println("Av " + name + " handleMessage SFRotation: " + rflt[0] + " " + rflt[1] + " " + rflt[2] + " " + rflt[3] );
		break;
	    case Field.SFSTRING:
		if(avatar == null ) break;
		try {
		    EventInSFBool trig = (EventInSFBool)avatar.getEventIn(((SFString)data).getValue());
		    trig.setValue( true );
		}
		catch(Exception e){
		    System.err.println(name+" err setting "+((SFString)data).getValue()+"\n\t"+e);
		}
		break;
	    default:
		throw new ConsumerException("Wrong field type for "+name);
	    }
	}
	catch(IllegalArgumentException e){
	    System.err.println("Error moving "+name+" "+e);
	}
    }

    public void update(){	
    }
   
    /**
    * New 11-12-05 gets values of position and rotation of the drone avatar and delivers
    * them to the UserAvatar so the user can beam to the drone
    */

   public void beamto( String target ){
if( target.equals(name) ){
   posflt = pos2.getValue(); rotflt = rot2.getValue();
   beam[0] = posflt[0]; beam[1] = posflt[1]; beam[2] = posflt[2];
   beam[3] = rotflt[0]; beam[4] = rotflt[1]; beam[5] = rotflt[2]; beam[6] = rotflt[3];
   avdeck.beamtocoord(beam);
 }
 } 

     public void userView( String target ){
  if(which!=null){
 if(bname_state){ which.setValue(-1); bname_state = false; }
 else { which.setValue(0); bname_state = true; }
  }
  avdeck.beamtoFinished();    
  }
  
  public void objectRoutes( String target ){
    float [] pflt = new float[3];
   float [] rflt = new float[4];
  if(bObject){
  //Browser browser = avdeck.getBrowser();
  Node PM = avdeck.getPrimeMover();
  if(bOR==true){
     avdeck.sharedObjectName( name, 1 );
     ((EventInSFVec3f)PM.getEventIn("set_start_position")).setValue( pos2.getValue() );
     pflt = pos2.getValue();
     System.out.println("Av " + name + " objectRoutes startPosition : " + pflt[0] + " " + pflt[1] + " " + pflt[2]   );
   rflt = rot2.getValue();
	 ((EventInSFRotation)PM.getEventIn("set_start_rotation")).setValue( rot2.getValue() );
	 System.out.println("Av " + name + " objectRoutes startRotation: " + rflt[0] + " " + rflt[1] + " " + rflt[2] + " " + rflt[3]  );
   //browser.addRoute( PM, "orientation_changed2", anchor, "translation");
   //browser.addRoute( PM, "position_changed2", anchor, "rotation" );
   bOR = false;
   }
  else {
	avdeck.sharedObjectName( "", 0 );
  // browser.deleteRoute( PM, "orientation_changed2", anchor, "translation");
   // browser.deleteRoute( PM, "position_changed2", anchor, "rotation");
   bOR = true;
      }
     }
   }

    public boolean getbAvatar(){
  return bAvatar;
     }

   public void setUserPosition( float[] fUP ){
  }

   public void checkPosition( float[] fUP, boolean bUPs  ){

   }


    public void delete() throws RuntimeException {
if(yak!=null){ texttalk.delete(); }
	Enumeration list = states.elements();
	while( list.hasMoreElements()){
	    postOffice.removeConsumer(((Route)list.nextElement()).getName());
} 
	states.removeAllElements();
 yak = null; field = null; field2 = null; which = null; anchor = avatar = null; 
    }
 }