/**
  Route.java

  Copyright (C) 1998 Geometrek

  This source is distributed as semi-opensource software. Please see the
  LICENSE file with this distribution or contact Geometrek (www.geometrek.com)
  for details.
*/

package matrix.deck.eai;

import matrix.RealTimeMessage;
import matrix.vrml.*;
import matrix.vrml.eai.*;
import matrix.deck.RealTimeConsumer;
import matrix.deck.ConsumerException;
import matrix.deck.Network;
import java.io.*;
import vrml.external.*;
import vrml.external.field.*;
import vrml.external.exception.*;

/**
 * manages a single VRML route. Now this is a Living Worlds' NetworkState node.
 *
 * @version 1.1
 */
public class Route implements RealTimeConsumer, EventOutObserver, Runnable {
    private String name;
    private Node myNode;
    private boolean continous = false, valChanged = false, echo = false, dead = false;
    private EventOut callingEvent;
    private EventIn fromnet;
    private Network postOffice;
	private long sleep = 400;
    private int stor = 25;
    private int iVn = 0;
    private Field val= null, lastVal = null;
    private int type;
    private Thread runner = null;
	public boolean bAvatar = false;
    
    public Route( Node netnode, Network office, int VN, long slp ) throws RuntimeException {
	postOffice = office;
	myNode = netnode;
	iVn = VN;
    sleep = slp;
	name = ((EventOutSFString)myNode.getEventOut("tag")).getValue();
	if(name.endsWith("_P")){
	stor = RealTimeMessage.POP;
     }
    if(name.endsWith("_F")){
    stor = RealTimeMessage.FOR;
    }
	continous = ((EventOutSFBool)myNode.getEventOut("cont")).getValue();
	echo = ((EventOutSFBool)myNode.getEventOut("echo")).getValue();
	callingEvent = myNode.getEventOut("value_tonet");  // GC workaround
 callingEvent.advise(this, new Integer(0) );
	fromnet = myNode.getEventIn("value_fromnet");
	type = (callingEvent.getType());
	try {
	    lastVal = EAITool.makeField( callingEvent );
	}
	catch(MatrixInvalidFieldException e){
	    System.err.println("Route "+name+" exeption in constructor "+e);
	}
	if( continous == true ){
	    runner = new Thread( this, name );
	    runner.start();
	}
    }
    
    public void setName( String name ){
	this.name = name;
    }

    public String getName(){
	return name;
    }
    
    public void handleMessage( String tag, Field data ) throws ConsumerException {
	if(!tag.equals(name))
	    throw new ConsumerException(name+" matches not "+tag);
	try {
	    EAITool.setEventIn( fromnet, data );
	    lastVal = data;
	}
	catch(Exception e){
	    throw new ConsumerException(name+"\n\t"+e);
	}
    }

public void userView( String target ){
}

public void objectRoutes( String target ){
 }

  public boolean getbAvatar(){
  return bAvatar;
     }

 public void setUserPosition( float[] flt ){
  }

  public void checkPosition( float[] fUP, boolean bUPs ){
   }

 public void beamto( String target ){
}

    public void update(){
    }

    public void run(){
	while( Thread.currentThread() == runner ) {
	    try{
		if( valChanged == true ){
		    postOffice.sendRealTime( stor, name, val, echo );
		    lastVal = val;
		    valChanged = false;
		}
		Thread.sleep(400);
	    }
	    catch(InterruptedException e){}
	    catch(IOException e){
		System.err.println("IOEx in route "+name+".run\n\t"+e);
	    }
	}
    }

    public void callback(EventOut value, double time, Object data) {
    if( dead == true ) return;
    try {
        if( value == null ) {
        System.err.println("Got null value from "+name);
        return;
        }
        if(!(value instanceof EventOut)) {
        System.err.println("value of class "+value.getClass().getName()+" from "+name);
        return;
        }
        if(value.getType()==type){
        val = EAITool.makeField( callingEvent );
        if( continous == false ){
        postOffice.sendRealTime( stor, name, val, echo );
        lastVal = val;
        valChanged = false;
        }
        else valChanged = true;
        }
    }
    catch(Exception e) {
        System.err.println("Error converting msg from "+data+":"+e); return;
    }
    }


    public void delete() throws RuntimeException {
	runner = null; 
if(callingEvent!=null){ callingEvent.unadvise(this); }
 fromnet = null; 
 val = lastVal = null;
 myNode = null;
 callingEvent = null;
 dead = true;
    }
}
