/**
  UserAvatar.java

   Copyright (C) 1998 Geometrek and 2005 in part by Russ Kinter

 *  This file is part of SET Matrix 1.
 *
 *  SET Matrix 1 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  SET Matrix 1 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package matrix.deck.eai;

import matrix.RealTimeMessage;
import matrix.vrml.*;
import matrix.vrml.eai.*;
import matrix.deck.RealTimeConsumer;
import matrix.deck.ConsumerException;
import matrix.deck.Network;
import matrix.deck.UserInterfaceListener;
import java.io.*;
import java.util.*;
import vrml.external.*; 
import vrml.external.field.*;
import vrml.external.exception.*;


/**
 *  manages Users avatar in local world, displays it in Ghost mode and sends updates to drones.
 *  also manages BOMU behaviours.
 *
 *  walking behaviour automation by Andrew Gerngross <agern@ix.netcom.com>.
 *
 *  @version 1.3
 */
public class UserAvatar implements  RealTimeConsumer, Runnable { 
   /**
    * New 11-12-05 Node for Chat
    */
    private EventInMFString yak;
    /**
    * New 11-12-05 String variables for keep track of avatar features
    */
    private String talk = "nah", human = "hah", inMotionBehaviour = null, notInMotionBehaviour = null;
    private long sleep = 400;
    /**
    * New 11-12-05 a is used as a boolean to determine if chat text is to be displayed.
    */
    private boolean bChat = false, bOR = true, bP = true, bO = true;
    /**
    * New 11-12-05 vn is viewernumber from EAIViewer class for which VRML plugin determination.
    */
    private int vn = 0, iO = -1, iP = -1;
     /**
    * New 11-12-05 newvp is for a Viewpoint that will bind when the user leaves ghost mode so the user wont go back to world entry viewpoint!
    * Not necessary in cosmoplayer!
    */
    private String newvp = "";
     /**
     * name is user' name. New 11-12-05 name_text is for users name display to be create VRML from String
     */
    private String name, name_text;
   /**
    * New 11-12-05 Beta is added as variable for beamto
    */
    private Beta avdeck;
    /**
     * Network for communicating 
     */ 
    private Network postOffice;
    /**
     *  Ghost mode state
     */
    private boolean show = false;
    private float Zaxis = (float)1.85;
	private float [] beam = new float[7];
   /**
    * New 11-12-05 position with beam to information
    */ 
    private float [] posflt = new float[3];
	/**
    * New 11-12-05 rotation with beam to information
    */ 
    private float [] rotflt = new float[4];
   /**
    * New 11-12-05 beamtostate needed to juggle which viewpoints to bind when leaving ghost mode. See hide and show methods  
    */
    public boolean beamtostate = false;
    public boolean bAvOk = true;
    /**
     * proxy holds the ProximitySensor from WorldObject class to get user location info.
     * New 11-12-05 vp holds the new Viewpoint to retain  where user's avatar stopped when user went into ghost mode. Again, not needed in Cosmoplayer
     */
    private Node proxy, anchor, avatar = null, vp = null;
    /**
     * New 11-12-05 fov holds variable vp's fieldOfView
     */
    private EventInSFFloat fov;
   /**
    * New 11-12-05 bind and bind2 are for viewpoints coordinated with boolean beamtostate. See hide and show methods
    */
    private EventInSFBool bind, bind2;
   /**
     * pos is the Avatar's translation exposedField
     *
     * New 11-12-05 vppos holds Viewpoint variable vp's position for changing when leaving ghost mode
     */ 
    private EventInSFVec3f pos, vppos, ghostVpPos;
    /**
    * New 11-12-05 eventIn where all the position and rotation info of beamto target avatar is given to the users av
    */
    private EventInMFFloat mffloat;
    /**
     * rot is the Avatar's rotation exposedField
     *
     * New 11-12-05 vpori holds Viewpoint variable vp's orientation for changing when leaving ghost mode
     */ 
    private EventInSFRotation rot, vpori, ghostVpRot;
    /**
     * Ghost mode Switch
     */
    private EventInSFInt32 which;
    private EventInSFString set_name = null;
    /**
     *  offset is for determining UserAvatars position in reference to ground
    */
    private SFVec3f offset = new SFVec3f((float)0.0,(float)-1.85,(float)0.0);
    private SFVec3f ghostset = new SFVec3f((float)0.0,(float)1.6,(float)5.0); 
    /**
     * posVal is the current position from the ProximitySensor with the offset calculated in
     *
     * New 11-12-05 posVal2 holds the current position when going into ghost mode. It is given to vppos when leaving ghost mode. Not used in Cosmoplayer!
     */ 
    private SFVec3f posVal, posVal2;
    /**
     * rotVal is current orientation from the ProximitySensor.
     *
     * New 11-12-05 rotVal2 holds the current orientation when going into ghost mode. It is given to vppos when leaving ghost mode. Not used in Cosmoplayer!
     */ 
    private SFRotation rotVal, rotVal2;
   /**
    * holds Routes for found Shared Event Protos
    */
    private Vector states = new Vector();

     /**
      * posChanged and rotChanged are used by the thread to determine if new UserAvatar location updates should be sent to network
      * dead means user is dead and updates are not to be done.
      */
    private boolean  dead = false; 
    private Thread runner = null;
     /**
      * holds the last time the Avatar moved
      */
    private java.util.Date lastMoved = null;  
	private posObserver po = null;
    private rotObserver ro = null;
    public boolean bAvatar = false;

 public class posObserver implements EventOutObserver {
 private boolean posChanged = false;
 private EventOutSFVec3f pos_out;
 private SFVec3f pos2 = new SFVec3f( 0, 0, 0 );
 private SFVec3f ghostpos = new SFVec3f( 0, 0, 0 );
 private boolean notnow = false;

    public posObserver( EventOutSFVec3f prox_pos ) throws Exception {
	 pos_out = prox_pos;
 pos_out.advise( this, new Integer(18) );
     }

   public void callback(EventOut value, double time, Object data) {
	if( show == true || dead == true )
	    return;
	try {
	    if( value == null ) {
		System.err.println("Got null value from "+name);
		return;
	    }
	    if(!(value instanceof EventOut)) {
		System.err.println("value of class "+value.getClass().getName()+" from "+name);
		return;
	    }
	
	    if(value.getType()==FieldTypes.SFVEC3F){
		   pos2 = offset.plus( new SFVec3f(pos_out.getValue()) );
          set_posChanged( true ); 
	    }
	}
	catch(Exception e) {
	    System.err.println("Error converting msg from "+name+"\n\t"+e);
	}
    }


public synchronized void set_posChanged( boolean tf ){
posChanged = tf;
}

public boolean get_posChanged(){
return posChanged;
}

public SFVec3f positionReturn(){
return pos2 ;}

public SFVec3f positionGhost(){
ghostpos = new SFVec3f(pos_out.getValue());
return ghostpos; }

public void delete(){                              
 pos_out.unadvise(this);	
 set_posChanged( false );
 pos_out = null;
    }
}
  public class rotObserver implements EventOutObserver {
 private  boolean rotChanged = false;
 private EventOutSFRotation rot_out;
 private SFRotation rot2 = new SFRotation(0, 1, 0, 0);
    public rotObserver( EventOutSFRotation prox_rot ) throws Exception {
	 rot_out = prox_rot;
  rot_out.advise( this, new Integer(12) );
     }

  public void callback(EventOut value, double time, Object data) {
	if( show == true || dead == true )
	    return;
	try {
	    if( value == null ) {
		System.err.println("Got null value from "+name);
		return;
	    }
	    if(!(value instanceof EventOut)) {
		System.err.println("value of class "+value.getClass().getName()+" from "+name);
		return;
	    }
	
	    if(value.getType()==FieldTypes.SFROTATION){
		   rot2 = new SFRotation(rot_out.getValue());
   set_rotChanged( true ); 
	    }
	}
	catch(Exception e) {
	    System.err.println("Error converting msg from "+name+"\n\t"+e);
	}
    }

public synchronized void set_rotChanged( boolean tf ){
rotChanged = tf;
}

public boolean get_rotChanged(){
return rotChanged;
}

public SFRotation orientationReturn(){
return rot2;}

public void delete(){
 rot_out.unadvise(this);
 set_rotChanged( false );	
 rot_out = null; 
    }
}

    /**
     * builds a new UserAvatar. It parses the loaded nodes and construct the needed scenegraph
     * and gets the events etc. Also builds all the necessary Route objects, if it is
     * a SharedObject.
     * New 11-12-05 the gesture list is now also used to list UserAvatar options 
     * (NG_chat_text,NG_humanoid,NG_no_lod) "NG_" prefix is for the UserAvatar
     * so the option does not show up as a gesture. 
     * NG_chat_text means the avatar is setup for chat text display.
     * NG_humanoid the avatar has Deep Matrix's humanoid animation methods: "inMotionBehaviour" 
     * and "notInMotionBehaviour"
     * NG_no_lod means the avatar does not want the built in LOD.
     *
     * One of the stranger aspects of UserAvatar is the use of EventInMFString wc in the constructor
     * This is a VRML eventin from the World class to display chat strings as VRML text. Moving this
     * eventIn World.class -> UserAvatar -> Beta  stops Cosmoplayer from crashing. Don't ask why just accept :)
     *
     * user_av_finished() is called from Beta when the user's avatar is finished.
     * drone_av_finished( false ) is called from Beta if the user's avatar throws an error.
     *
     * @param object the loaded object, containing the name and a Transform
     * @param browser reference to the VRML browser
     * @param postOffice reference to the Network object
     *
     
     */
    public UserAvatar( SO1 object, Network post, Browser browser, Node prox, Beta deck, int VIN, float diff, long slp )
	throws RuntimeException {
 System.out.println("UserAvatar started");
 Node [] Foo = null;
 Node [] xform = null;
 Node [] nodes = null;
 Zaxis = diff;
	name = object.getName();
	postOffice = post;
	proxy = prox;
 avdeck = deck;
 vn = VIN;
 sleep = slp;
 //offset = new SFVec3f(0,-diff,0);
   try {
 bind = (EventInSFBool)proxy.getEventIn("set_bind");
}
catch(InvalidEventInException e){ System.out.println("No set_bind: "+ e); }
catch(NullPointerException f){ System.out.println("No set_bind: "+ f); }
try {
 mffloat = (EventInMFFloat)proxy.getEventIn("setBeamTo");
}
catch(InvalidEventInException e){ System.out.println("No setBeamTo: "+ e); }
catch(NullPointerException f){ System.out.println("No setBeamto: "+ f); }
 System.out.println("UA Avalon! bind got!");
      try {
 po = new posObserver((EventOutSFVec3f)proxy.getEventOut("position_changed"));
 }
catch(Exception e){System.out.println("UA position_changed error: " + e);}
  try {
 ro = new rotObserver( (EventOutSFRotation)proxy.getEventOut("orientation_changed"));
 }
catch(Exception e){System.out.println("UA orientation_changed error: " +e);}
 name_text = ( "Transform { " + 
						       "translation 0 3.75 0 " +
						       "children [ " + 
             "LOD {  " +
             "range 20 " +                                         
             "level [ " +
						       "Billboard {   " +
						       "children [  " +
						       "Shape { geometry Text { string [\"" + name.substring(1) + "\"] maxExtent 4  " +
						       "fontStyle FontStyle { family [\"SANS\"] justify \"MIDDLE\" size .5  } } " +
						       "} " +
             "]} " +
             "Group {} " +                                          
						       "]}]}  " );
	// get nodes
	try {
 xform = object.getTransforms2();
    }
  catch(NullPointerException np){ System.out.println( "object.getTransforms(): " + np );
   avdeck.drone_av_finished( false );
	    return;}
if(xform[0].getType().equals("Avatar2")){
 System.out.println("UserAvatar Avatar2");
  	try {
 Foo = object.getTransforms1();
 nodes = ((EventOutMFNode)Foo[0].getEventOut("children_changed")).getValue();
    }
  catch(NullPointerException np){ System.out.println( "object.getTransform(): " + np );
  avdeck.drone_av_finished( false );
	    return; 
     }
	// check for SharedObject definition
  for( int i = 0; i < nodes.length; i++ ) {
    System.out.println("UA Node Type is: "+ nodes[i].getType());
	if( nodes[i].getType().equals("SharedObject")){
	    Node soNode = nodes[i];
	    Node[] statesVal = ((EventOutMFNode)soNode.getEventOut("states_changed")).getValue();
	    if( statesVal.length != 0 ){
		for( int j = 0; j < statesVal.length; j += 1 ){
		   if(!deck.bLogOut){
		    if( statesVal[j].getType().startsWith("Network")){
			Route route = new Route( statesVal[j], postOffice, VIN, sleep );
			route.setName( name+"_"+route.getName());
			postOffice.addConsumer( route );
			states.addElement( route );
		    }
      }
	
          }
		 }
	    if(!name.endsWith("_Obj")){ name = name + "_Obj" ;}
	    } 
   
if( nodes[i].getType().equals("Avatar")){
	System.out.println( "Is Avatar" ); 
  avatar = nodes[i];
try {
EventOut field = nodes[i].getEventOut("gestures");
	    // workaround for Blaxxun Contact Beta bug
	    // getEventOut should throw Exception not return null
	    if( field == null )
		throw new InvalidEventOutException();
	    if( field.getType() != FieldTypes.MFSTRING )
		throw new InvalidEventOutException();
  int b = (((EventOutMFString)field).getValue()).length;
   String all [] = new String[b];
   all = (((EventOutMFString)field).getValue());
     for (int c = 0; c<=b; c++){ if(c<b){
 if( all[c].equals("NG_chat_text")){talk = all[c]; }
 if( all[c].equals("NG_humanoid")){human = all[c]; }}
 if(c==b){
     avdeck.addBehavs(all); 
	    
     }}
	}
	    // workaround for Blaxxun Contact Beta bug
	    // getEventOut should throw Exception not return null
catch(InvalidEventOutException e){
	    avatar = null; bChat = false; 
     bAvOk = false;
     System.out.println(name+" is not BOMU VRML");
     avdeck.drone_av_finished( false ); 
    return;
	}
	// this bit is for Cortona which throws a NullPointer          
	// instead of an InvalidEventOut  !
	catch(NullPointerException e){
    avatar = null; ;  
    bAvOk = false; 
    System.out.println(name+" is not BOMU VRML" );
    avdeck.drone_av_finished( false );
    return; }
if(avatar!=null&&talk.equals("NG_chat_text")){ 
 System.out.println("UA: Av is talker!");
try { 
  yak = (EventInMFString)nodes[i].getEventIn("talker_text");
   bChat = true;
   name_text = "Group {} "; 
    }
		catch(InvalidEventInException e){
	    avatar = null; 
     bChat = false;
     bAvOk = false;
     System.out.println(name+" is not valid VRML");
     avdeck.drone_av_finished( false );
     return;
	}
	// this bit is for Cortona which throws a NullPointer          
	// instead of an InvalidEventOut  !
	catch(NullPointerException e){
	    avatar = null; 
    bChat = false;
     bAvOk = false;
     System.out.println(name+" is not valid VRML");
     avdeck.drone_av_finished( false );
     return;
	}
 try { 
 set_name = (EventInSFString)nodes[i].getEventIn("set_name");
   bChat = true;
   name_text = "Group {},";
   }
		catch(InvalidEventInException e){
       bChat = false; avatar = null;
     bAvOk = false;
      System.out.println(name+" is not valid VRML");
     avdeck.drone_av_finished( false );
	    return;
	   }
	// this bit is for Cortona which throws a NullPointer          
	// instead of an InvalidEventOut  !
	catch(NullPointerException e){
    System.out.println(name+" is not valid VRML");
     bChat = false;
     avatar = null;
     bAvOk = false;
    avdeck.drone_av_finished( false );   
     return;
	    }
   avdeck.chatDisplay( true );
  }
if(avatar!=null&&human.equals("NG_humanoid")){ 
	    try {
		inMotionBehaviour = ((EventOutSFString)(nodes[i].getEventOut("inMotionBehaviour"))).getValue();
		notInMotionBehaviour = ((EventOutSFString)(nodes[i].getEventOut("notInMotionBehaviour"))).getValue();
	    }
        catch (InvalidEventOutException d ){
	  inMotionBehaviour = null;
		notInMotionBehaviour = null;
 bChat = false;
  avatar = null;
  bAvOk = false;
  avdeck.drone_av_finished( false );
  System.out.println("avatar does not define motion behaviours" + d );
  return;
   }
   catch (NullPointerException f ){
	  inMotionBehaviour = null;
		notInMotionBehaviour = null;
 bChat = false;
  avatar = null;
  bAvOk = false;
  avdeck.drone_av_finished( false );
  System.out.println("avatar does not define motion behaviours" + f );
  return;
   }
	    catch (Exception ex) {
		inMotionBehaviour = null;
		notInMotionBehaviour = null;
 bChat = false;
  avatar = null;
  bAvOk = false;
  avdeck.drone_av_finished( false );
		System.out.println("avatar does not define motion behaviours" );
   return;
	    }	
}
}
}
if(bAvOk==true){
Node [] help3 = browser.createVrmlFromString( name_text );
try {
  vp = xform[1];
}
catch(InvalidNodeException ine ){ System.out.println( "UA viewpoint node: " + ine );
avdeck.drone_av_finished( false );  
return;}
catch(NullPointerException np){ System.out.println( "UA no viewpoint node: " + np );
avdeck.drone_av_finished( false ); 
return;}
 if(vp!=null){ 
 if(vp.getType().equals("Viewpoint")){
  fov = (EventInSFFloat)vp.getEventIn("set_fieldOfView");
 bind2 = (EventInSFBool)vp.getEventIn("set_bind");
 vppos = (EventInSFVec3f)vp.getEventIn("set_position");
	vpori = (EventInSFRotation)vp.getEventIn("set_orientation");
 System.out.println( "Viewpoint got!" );
}
else { System.out.println( "vp is not Viewpoint Node: " + vp.getType() ); }
}
 ((EventInMFNode)xform[0].getEventIn("name_tag")).setValue( help3 );
 which = (EventInSFInt32)xform[0].getEventIn("which");
	// get events
	pos = (EventInSFVec3f)xform[0].getEventIn("translation");
	rot = (EventInSFRotation)xform[0].getEventIn("rotation");
// mffloat = (EventInMFFloat)xform[0].getEventIn("set_mffloat");
//	bind = (EventInSFBool)xform[0].getEventIn("bind"); 
  anchor = xform[0];
 which.setValue(-1);
// call back to WorldObject to reset the current Viewpoint in the WorldObject.
 avdeck.setViewpoint();
// get update links
 posVal = po.positionReturn();
pos.setValue(posVal.getValue());
po.set_posChanged( false ); 
rotVal = ro.orientationReturn();
rot.setValue(rotVal.getValue());
ro.set_rotChanged( false );
 runner = new Thread( this );
 runner.start();
    }
  }
else { 
avdeck.drone_av_finished( false );
return; 
  } 
}

    /**
     * shows the Avatars geometrey and binds the Ghost Viewpoint.
     */

    public void show(){
 which.setValue(0);
 posVal2 = po.positionGhost();
 rotVal2 = new SFRotation(ro.orientationReturn());
 beamtostate = false; 
	show = true;
	bind.setValue(true);
 //ghostVpPos.setValue( ghostset.plus( posVal2 ).getValue() );
 //ghostVpRot.setValue( rotVal2.getValue() );
    }

    /**
     * reverses the action of show.
     * New 11-12-05 hide method will now set the user's avatar's location to
     * where the user left it when going into OBE or Ghost mode. Previous versions
     * put the user back at the world entry viewpoint.
     */

    public void hide(){                                
  show = false;
	which.setValue(-1);
if(vn!=3){
 vppos.setValue(posVal2.getValue());
 vpori.setValue(rotVal2.getValue());
bind.setValue(false);
if(beamtostate==false){	bind2.setValue(true); } else { bind.setValue(false); }}
    }

     public void beamover( float[] beamdata ){
     float [] ft = { beamdata[0],beamdata[1]+Zaxis,beamdata[2],beamdata[3],beamdata[4],beamdata[5],beamdata[6] };
 //  this.hide(); 
	 show = false;
	which.setValue(-1);
 //  posVal2 =  new SFVec3f(beamdata[0],beamdata[1],beamdata[2]);
 //  rotVal2 = new SFRotation(beamdata[3],beamdata[4],beamdata[5],beamdata[6]);
   mffloat.setValue(ft);
   avdeck.beamtoFinished();
}
    /**
     * sets correct offset from NavigationInfo so that Avatar at others clients do the
     * right thing.
     */

    public void setOffset( float off ){
	offset = new SFVec3f( 0, -off, 0);
    }

    public String getName(){
	return name;
    }

	   public void chatDisplay ( String [] foo ){
 if(yak!=null){ yak.setValue(foo); }
    }

    public void handleMessage( String tag, Field data ) throws ConsumerException {
	if(!tag.equals(name))
	    throw new ConsumerException(name+" matches not "+tag);
	try {
	    switch(data.getType()){
	    case Field.SFVEC3F:
		pos.setValue(((SFVec3f)data).getValue());
		break;
	    case Field.SFROTATION:
		rot.setValue(((SFRotation)data).getValue());
		break;
	    case Field.SFSTRING:
		if( avatar == null ) break;
		try {
		    EventInSFBool trig = (EventInSFBool)avatar.getEventIn(((SFString)data).getValue());
		    trig.setValue( true );
		}
		catch(Exception e) {
		    System.err.println(name+" err setting "+((SFString)data).getValue()+"\n\t"+e);
		}
		break;
	    default:
		throw new ConsumerException("Wrong field type for "+name);
	    }
	}
	catch(IllegalArgumentException e){System.err.println("Wrong type in "+name+"\n\t"+e);}
    }

   public void update(){
	try {
	    if( posVal != null ){
		postOffice.sendRealTime( RealTimeMessage.NONE, name, posVal, false );
		po.set_posChanged( false );
	
	    }
	    if( rotVal != null ){
		postOffice.sendRealTime( RealTimeMessage.NONE, name, rotVal, false );
	    ro.set_rotChanged( false );
	    }
	}
	catch(IOException e){ System.err.println("IO in "+name+".update\n\t"+e); }

	Enumeration list = states.elements();
	while( list.hasMoreElements()){
	    ((Route)list.nextElement()).update();
	}
    } 

    public void delete() throws RuntimeException {
	runner = null;
	dead = true;
	 po.delete();
	ro.delete();
Enumeration list = states.elements();
	while( list.hasMoreElements()){
	    postOffice.removeConsumer(((Route)list.nextElement()).getName());
	}  yak = null;vp = proxy = anchor = avatar = null; 
    }

 /**
  * New 11-12-05 Here to satisfy need of interface
  */

  public void beamto( String target ){
 if( target.equals(name) ){
   posflt = po.positionReturn().getValue(); rotflt = ro.orientationReturn().getValue();
   beam[0] = posflt[0]; beam[1] = posflt[1]; beam[2] = posflt[2];
   beam[3] = rotflt[0]; beam[4] = rotflt[1]; beam[5] = rotflt[2]; beam[6] = rotflt[3];
   avdeck.beamtocoord(beam);
  }
}
  
 public float [] getUserPosition_Rotation(){
   posflt = po.positionReturn().getValue(); rotflt = ro.orientationReturn().getValue();
   beam[0] = posflt[0]; beam[1] = posflt[1]; beam[2] = posflt[2];
   beam[3] = rotflt[0]; beam[4] = rotflt[1]; beam[5] = rotflt[2]; beam[6] = rotflt[3];
   return beam;
 }

    public void userView( String target ){
 }
  
   public void objectRoutes( String target ){
if(name.endsWith("_Obj")){
Node PM = avdeck.getPrimeMover();
Browser browser = avdeck.getBrowser();
  if(bOR==true){
   ((EventInSFBool)proxy.getEventIn("enabled")).setValue( false );
   po.set_posChanged( false );
   ro.set_rotChanged( false );
   browser.addRoute( PM, "orientation_changed", anchor, "rotation");
   browser.addRoute( PM, "position_changed", anchor, "translation" );
  bOR = false;
  }
  else {
   browser.deleteRoute( PM, "orientation_changed", anchor, "rotation" );
   browser.deleteRoute( PM, "position_changed", anchor, "translation" );
((EventInSFBool)proxy.getEventIn("enabled")).setValue( true );
   bOR = true;
   }
  }
 }
  
   public boolean getbAvatar(){
  return bAvatar;
     }

  public void setUserPosition( float[] flt ){
  }

     public void checkPosition( float[] fUP, boolean bUPs  ){
   }

 /**
 * New 11-12-05 When run starts it tells Beta (avdeck) that the UserAvatar is done
 * by calling "user_av_finished" If the user's avatar is optioned for chat text display,
 * it will send the EventInMFString yak to Beta.
 * A big change is the sleep time increased from 100 to 400 milliseconds. Much easier on the network!
 *
 */
    public void run(){
   if(bChat&&set_name!=null){
  set_name.setValue(name.substring(1));
   }
avdeck.user_av_finished();
	while( Thread.currentThread() == runner ) {
 boolean moving = false;
	    try { 
		if( po.get_posChanged() ){
      moving = true;
      posVal = po.positionReturn();
		    postOffice.sendRealTime( RealTimeMessage.NONE, name, posVal, false );
      pos.setValue(posVal.getValue());
		    po.set_posChanged( false ); 
		}
		if( ro.get_rotChanged() ){
      moving = true;
      rotVal = ro.orientationReturn();
		    postOffice.sendRealTime( RealTimeMessage.NONE, name, rotVal, false );
      rot.setValue(rotVal.getValue());
		    ro.set_rotChanged( false ); 
		}
 		// by Andrew Gerngross
		if (inMotionBehaviour != null ) {
		    if ( moving ) {
			if (lastMoved == null) {
			    lastMoved = new java.util.Date();
			    this.trigger( inMotionBehaviour );
			}
		    } else {
			if (lastMoved != null) {
			    lastMoved = null;
			    this.trigger( notInMotionBehaviour );
			}
		    }
		}
		// end
		Thread.sleep(sleep);
	    }
	    catch(InterruptedException e){}
	    catch(IOException e){
		System.err.println("IOEx in "+name+".run\n\t"+e);
	    }
	}
    }

 /**
 * New 11-12-05 beamover method sets the position and orientation from the target 
 * Avatar to the UserAvatar. Everything afterwards is done by javascript/vrmlscript 
 * inside the Avatar Proto.
 *  
 */
 	public void gesture( int gest ){
	try {
	    postOffice.sendRealTime( RealTimeMessage.NONE, name, new SFInt32( gest ), true );
	}
	catch(IOException e) {
	    System.err.println(name+" error posting "+gest+"\n\t"+e);
	}
   }

    /**
     * posts a behaviour event.
     *
     * @param behab the name of the behaviour
     */
    private void trigger( String behab ){
	try {
	    postOffice.sendRealTime( RealTimeMessage.NONE, name, new SFString( behab ), true );
	}
	catch(IOException e) {
	    System.err.println(name+" error posting "+behab+"\n\t"+e);
	}
    }

}
