/**
  PrimeRoute.java

  Copyright (C) 1999 Geometrek	and 2009 by R. Russell Kinter
 
 *  This file is part of DeepMatrixIP9.
 *
 *  DeepMatrixIP9 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrix IP9 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 

package matrix.deck.eai2;

import matrix.RealTimeMessage;
import matrix.vrml.*;
import matrix.vrml.eai2.*;
import matrix.deck.RealTimeConsumer;
import matrix.deck.ConsumerException;
import matrix.deck.Network;
import matrix.deck.CretanString;
import java.io.*;
import vrml.eai.*;
import vrml.eai.field.*;
import vrml.eai.event.*;

/**
 * manages a single VRML route. Now this is a Living Worlds' NetworkState node.
 *
 * @version 1.1
 */
public class PrimeRoute implements RealTimeConsumer, Runnable {
    private String target = null;
	private String beaming = null;
    private String username = null;
    private String name = "PM";
    private Node myNode;
    private CretanString PMCS = new CretanString();
    private boolean bActive = false, valChanged = false, echo = false, dead = false;
    private EventInSFRotation setRot = null;
    private EventInSFVec3f setPos = null;
    private EventInMFFloat setBeamTo = null;
    private Network postOffice;
	private long sleep = 400;
    private float [] fDefault = { (float)0.0, (float)0.0, (float)0.0, (float)0.0, (float)1.0, (float)0.0, (float)0.0 };
    private int iVn = 0;
    private Field val= null, lastVal = null;
    private int type;
    private int iState = 0;
    private posPMObserver PPMO = null;
    private rotPMObserver RPMO = null;
	private savePMObserver SPMO = null;
    private Thread runner = null;
	public boolean bAvatar = false;

   public class posPMObserver implements VrmlEventListener {
 public boolean posChanged = false;
 private EventOutSFVec3f pos_out;
 private SFVec3f pos = new SFVec3f( 0, 0, 0 );
 private boolean notnow = false;

    public posPMObserver( EventOutSFVec3f prox_pos ) throws Exception {
	 pos_out = prox_pos;
      pos_out.addVrmlEventListener(this);
     }

   	public void eventOutChanged( VrmlEvent vrmlevent ) {

  BaseField bf =  vrmlevent.getSource();
		if( bf.getType()==11 ) {
     pos = new SFVec3f( pos_out.getValue() );
	 set_posChanged( true );
    }
   else
    return;
  }

public synchronized void set_posChanged( boolean tf ){
posChanged = tf;
}

public boolean get_posChanged(){
return posChanged;
}
  
public SFVec3f positionReturn(){
return pos ;}


public void delete(){                              
 pos_out.removeVrmlEventListener(this);	
 set_posChanged( false );
 pos_out = null;
    }
}

  public class rotPMObserver implements VrmlEventListener {
 private  boolean rotChanged = false;
 private EventOutSFRotation rot_out;
 private SFRotation rot = new SFRotation(0, 1, 0, 0);

    public rotPMObserver( EventOutSFRotation prox_rot ) throws Exception {
	 rot_out = prox_rot;
  rot_out.addVrmlEventListener(this);
     }

	public void eventOutChanged( VrmlEvent vrmlevent ) {
  BaseField bf =  vrmlevent.getSource();
		if( bf.getType()==7 ) {
   rot = new SFRotation(rot_out.getValue());
   set_rotChanged( true );  
		}
	 else
    return;
   }

public synchronized void set_rotChanged( boolean tf ){
rotChanged = tf;
}

public boolean get_rotChanged(){
return rotChanged;
}

public SFRotation orientationReturn(){
return rot;}

public void delete(){ 
 rot_out.removeVrmlEventListener(this);
set_rotChanged( false );	
 rot_out = null; 
    }
}

  public class savePMObserver implements VrmlEventListener {
 private  boolean saveChanged = false;
 private EventOutSFInt32 save_out;
 private int saved = 0;

    public savePMObserver( EventOutSFInt32 saver ) throws Exception {
	 save_out = saver;
  save_out.addVrmlEventListener(this);
     }

	public void eventOutChanged( VrmlEvent vrmlevent ) {
  BaseField bf =  vrmlevent.getSource();
 if( bf.getType()==BaseField.SFInt32 ) {
   saved = save_out.getValue();
   if(saved==1){
    try {
  float [] mft = new float[7]; 
  float [] tft = new float[3];
  float [] rft = new float[4];
SFVec3f sfv = PPMO.positionReturn();
tft = sfv.getValue();
SFRotation sfr = RPMO.orientationReturn();
rft = sfr.getValue();
mft[0] = tft[0];
mft[1] = tft[1];
mft[2] = tft[2];
mft[3] = rft[0];
mft[4] = rft[1];
mft[5] = rft[2];
mft[6] = rft[3];
MFFloat mff = new MFFloat( mft );
SFInt32 sfi = new SFInt32( 1 );
  postOffice.sendRealTimeSave( target, mff, sfi );
	    }
    catch(InterruptedException f){}
	catch(IOException e){ System.err.println("IO in "+name+".sendRealTimeSave\n\t"+e); return;}
	  set_saveChanged( true ); 
      saved = 0;
      }
  if(saved==2){
    try {
  if(beaming!=null){
   postOffice.sendRealTime( RealTimeMessage.NONE, beaming, PPMO.positionReturn(), true );
   postOffice.sendRealTime( RealTimeMessage.NONE, beaming, RPMO.orientationReturn(), true );
    }
     }
catch(IOException e){
		System.err.println("IOEx in "+name+" object beaming to user: \n\t"+e);
	    }
   }
     }
    else
    return;
   }

public synchronized void set_saveChanged( boolean tf ){
saveChanged = tf;
}

public boolean get_saveChanged(){
return saveChanged;
}

public int saveReturn(){
return saved;
}

public void delete(){ 
 save_out.removeVrmlEventListener(this);
set_saveChanged( false );	
 save_out = null; 
    }
}
    
    public PrimeRoute( Node pm, Network office, int VN, long slp, String username ) throws RuntimeException {
	this.username = username;
    postOffice = office;
	myNode = pm;
	iVn = VN;
    sleep = slp;
    //PMCS.setCretan( false );
    PMCS.setString("WAIT");
    try {
	PPMO = new posPMObserver( (EventOutSFVec3f)myNode.getEventOut("position_changed") );
	RPMO = new rotPMObserver( (EventOutSFRotation)myNode.getEventOut("orientation_changed") );
    setRot = (EventInSFRotation)myNode.getEventIn("set_rotation2");
    setPos = (EventInSFVec3f)myNode.getEventIn("set_position2");
    setBeamTo = (EventInMFFloat)myNode.getEventIn("set_BeamTo");
	SPMO = new savePMObserver( (EventOutSFInt32)myNode.getEventOut("save_changed") );
     }
    catch(NullPointerException f){ System.out.println( "CtrlRoute error: " + f ); }
    catch(InvalidEventInException g){System.out.println( "CtrlRoute error: " + g );  }
    catch(InvalidEventOutException h){System.out.println( "CtrlRoute error: " + h ); } 
    catch(Exception k){System.out.println( "CtrlRoute error: " + k ); }
   
	    runner = new Thread( this, "PrimeMover" );
	    runner.start();
    }
    
    public void setName( String name ){
	this.name = name;
    }

    public String getName(){
	return name;
    }
    
    public void handleMessage( String tag, Field data ) throws ConsumerException {
	if(!tag.equals(name))
	    throw new ConsumerException(name+" matches not "+tag);
		try {
	    switch(data.getType()){
	    case Field.SFVEC3F:
    //System.out.println("PR hM SFVec3f received");
		setPos.setValue(((SFVec3f)data).getValue());
		break;
	    case Field.SFROTATION:
    //System.out.println("PR hM SFRotation received");
		setRot.setValue(((SFRotation)data).getValue());
		break;
	    default:
		throw new ConsumerException("Wrong field type for "+name);
	    }
	}
	catch(IllegalArgumentException e){System.err.println("Wrong type in "+name+"\n\t"+e);
    }
	catch(Exception e){
	    throw new ConsumerException(name+"\n\t"+e);
	}
    }

public void userView( String target ){
    synchronized(PMCS) { 
    if(target.equals("WAKE")){ 
    	PMCS.setString("WAKE");
	//	PMCS.setCretan( true );
      }
    else {
	   if(target.equals("WAIT")){
        PMCS.setString(target);
     //  PMCS.setCretan( false ); 
      }
    }
     PMCS.notify();  
    }
   }
   
public void objectName( String target ){
}

public void objectRoutes( String target ){
 }

  public boolean getbAvatar(){
  return bAvatar;
     }

  public void setUserPosition( float[] flt ){
  }

  public void checkPosition( float[] fUP, boolean bUPs  ){
 }
 
public void beamto( String target ){
}

    public void update(){
     if(bActive){
	 	try {
	    if( PPMO.get_posChanged() ){
		postOffice.sendRealTime( RealTimeMessage.NONE, target, PPMO.positionReturn(), false );
		PPMO.set_posChanged( false ); 
	    }
	    if( RPMO.get_rotChanged() ){
		postOffice.sendRealTime(RealTimeMessage.NONE, target, RPMO.orientationReturn(), false );
		RPMO.set_rotChanged( false );
	    }
	}
	catch(IOException e){ System.err.println("IO in "+name+".update\n\t"+e); }
      }
    }

 public Node getPrimeMover(){
  return myNode;
  }

private synchronized void setTargetState( String target, int state ){
this.target = target; iState = state;
}

public void sharedObjectName( String target, int state ){
 setTargetState( target, state );
 }

public void beamingObjectName( String target, float [] mft, int state ){
// System.out.println("PrimeRoute beamingObjectName state: " + state );
 beaming = target; setTargetState( target, state );
if(0<iState){ 
if(setBeamTo!=null){ setBeamTo.setValue( mft ); }
// else { System.out.println("PM beamingObjectName failed eventIn setBeamTo is null!");}
}
else {
MFFloat mff = new MFFloat( fDefault );
SFInt32 sfi = new SFInt32( 0 );
  try {
  postOffice.sendRealTimeSave( target, mff, sfi );
   }
catch(InterruptedException f){}
catch(IOException e){ System.out.println("PM sendRealTimeSave: " + e );	   }
// System.out.println("PrimeRoute beamingObjectName sendRealTimeSave" + state );
}
 }
	 
   private void runControl(){
	   synchronized(PMCS) { 
	  if(PMCS.getString().equals("WAIT")) {
       try {
        PMCS.wait();
         }
	  catch( InterruptedException e ){}
       }
       else 
	  if(PMCS.getString().equals("WAKE")) {
         try {
		PMCS.wait(sleep);
          }
    	catch( InterruptedException e ){}
        }
      else
      if(PMCS.getString().equals("KILL")){
        runner = null;
        }
       }
      }

    
    public void run(){
	while( Thread.currentThread() == runner ) {
	     try { 
		if( PPMO.get_posChanged() ){
		    postOffice.sendRealTime( RealTimeMessage.NONE, target, PPMO.positionReturn(), true );
		    PPMO.set_posChanged( false ); 
		}
		if( RPMO.get_rotChanged() ){
		    postOffice.sendRealTime( RealTimeMessage.NONE, target, RPMO.orientationReturn(), true );
		    RPMO.set_rotChanged( false ); 
         }
		}
	    catch(IOException e){
		System.err.println("IOEx in route "+name+".run\n\t"+e);
	    }
        runControl();
	  }
    }

    public void delete() throws RuntimeException {
	 if(PPMO!=null){ PPMO.delete(); }
     if(RPMO!=null){ RPMO.delete(); } 
     myNode = null;
	synchronized(PMCS) {  
     PMCS.setString("KILL");
     PMCS.notify();  
      }
    }
}
