/**
  Route.java

Copyright (C) 1999 Geometrek	and 2009 by R. Russell Kinter
 
 *  This file is part of DeepMatrixIP9.
 *
 *  DeepMatrixIP9 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrix IP9 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */  

package matrix.deck.eai2;

import matrix.RealTimeMessage;
import matrix.vrml.*;
import matrix.vrml.eai2.*;
import matrix.deck.RealTimeConsumer;
import matrix.deck.ConsumerException;
import matrix.deck.Network;
import java.io.*;
import vrml.eai.*;
import vrml.eai.field.*;
import vrml.eai.event.*;

/**
 * manages a single VRML route. Now this is a Living Worlds' NetworkState node.
 *
 * @version 1.1
 */
public class Route implements RealTimeConsumer, VrmlEventListener, Runnable {
    private String name;
    private Node myNode;
    private boolean continous = false, valChanged = false, echo = false, dead = false;
    private EventOut callingEvent;
    private EventIn fromnet;
    private Network postOffice;
	private long lSleep = 400;
    private long  lTime = 0;
    private int stor = 25;
    private boolean persist = false;
    private int iVn = 0;
    private Field val= null, lastVal = null;
    private int type;
    private Thread runner = null;
	public boolean bAvatar = false;
    
    public Route( Node netnode, Network office, int VN, long slp ) throws RuntimeException {
	postOffice = office;
	myNode = netnode;
	iVn = VN;
    lSleep = slp;
	name = ((EventOutSFString)myNode.getEventOut("tag")).getValue();
	if(name.endsWith("_P")){
	stor = RealTimeMessage.POP;
     }
    if(name.endsWith("_F")){
    stor = RealTimeMessage.FOR;
    }
	if(name.endsWith("_C")){
    stor = RealTimeMessage.CLR;
    }
	continous = ((EventOutSFBool)myNode.getEventOut("cont")).getValue();
	echo = ((EventOutSFBool)myNode.getEventOut("echo")).getValue();
	callingEvent = myNode.getEventOut("value_tonet");  // GC workaround
 callingEvent.addVrmlEventListener(this);
	fromnet = myNode.getEventIn("value_fromnet");
	type = (callingEvent.getType());
	try {
	    lastVal = EAITool.makeField( callingEvent );
	}
	catch(MatrixInvalidFieldException e){
	    System.err.println("Route "+name+" exeption in constructor "+e);
	}
	if( continous == true ){
	    runner = new Thread( this, name );
	    runner.start();
	}
    }
    
    public void setName( String name ){
	this.name = name;
    }

    public String getName(){
	return name;
    }
    
    public void handleMessage( String tag, Field data ) throws ConsumerException {
	if(!tag.equals(name))
	    throw new ConsumerException(name+" matches not "+tag);
	try {
	    EAITool.setEventIn( fromnet, data );
	    lastVal = data;
	}
	catch(Exception e){
	    throw new ConsumerException(name+"\n\t"+e);
	}
    }

 public void userView( String target ){
  }

 public void objectRoutes( String target ){
  }

 public boolean getbAvatar(){
  return bAvatar;
     }

 public void setUserPosition( float[] flt ){
  }

 public void checkPosition( float[] fUP, boolean bUPs  ){
 }

 public void beamto( String target ){
}

    public void update(){
    }

    private synchronized void set_valChanged( boolean tf ){
    valChanged = tf;
    }

    public void run(){
	while( Thread.currentThread() == runner ) {
	    try{
		if( valChanged == true ){
		    postOffice.sendRealTime( stor, name, val, echo );
		    lastVal = val;
		    set_valChanged ( false );
		}
		Thread.sleep(lSleep);
	    }
	    catch(InterruptedException e){}
	    catch(IOException e){
		System.err.println("IOEx in route "+name+".run\n\t"+e);
	    }
	}
    }

    public void eventOutChanged(VrmlEvent vrmlevent) {
	if( dead == true ) return;
 //long lTime2 = System.currentTimeMillis();
 //  if( lTime2 < (lTime + lSleep)  ){ return;  }
 //  else {
BaseField bf =  vrmlevent.getSource();
 try {
	    if( type != bf.getType() ) {
		System.err.println("Got wrong value from "+name);
		return;
	    }
	    val = EAITool.makeField( callingEvent );
	    if( continous == false ){
		postOffice.sendRealTime( stor, name, val, echo );
		lastVal = val;
		set_valChanged ( false ); 
	    }
	    else set_valChanged ( true );
	}
	catch(IOException f){System.err.println("Route: " + f);return;}
	catch(Exception e) {
	    System.err.println("Error converting msg from "+bf+":"+e); return;
	}
//  lTime = lTime2; }
    }

    public void delete() throws RuntimeException {
	runner = null; 
if(callingEvent!=null){ callingEvent.removeVrmlEventListener(this); }
 fromnet = null; 
 val = lastVal = null;
 myNode = null;
 callingEvent = null;
 dead = true;
    }
}
