/*
  WorldSendChat.java

  Copyright (C) 2009 by R. Russell Kinter
 
 *  This file is part of DeepMatrixIP9.
 *
 *  DeepMatrixIP9 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrix IP9 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */









package matrix.deck.eai2;

import matrix.vrml.*;
import java.util.*;
import vrml.eai.*;
import vrml.eai.field.*;
import vrml.eai.event.*;

/**
 * New 11-12-05 TimerEvent duplicates the TimerSensor VRML Node for sending events at discrete intervals
 * The difference is the thread sleeps while a TimeSensor runs continously. We therefore hope to save processing power!
 * Instanced by WorldObject class.
 * @version 0.1    
 * @author Russ Kinter
 */

public class TimerEvent implements VrmlEventListener, Runnable {
    
  private EventOutSFBool sfActiveOut;
  private EventInSFBool sfActiveIn;
   

  private long lSleep = 300;
    /**
     *  New 11-12-05 runner controls the rate of how many strings go to Beta deck and the network.
     */
  private boolean bSleep = true;
  private Thread runner = null;



  public  TimerEvent( Node WSC ){   
  sfActiveOut = (EventOutSFBool)WSC.getEventOut("active_changed");
  sfActiveOut.addVrmlEventListener(this);
  sfActiveIn =	(EventInSFBool)WSC.getEventIn("set_active");
  lSleep = new Float( ((EventOutSFFloat)WSC.getEventOut("sleep_time")).getValue() ).longValue();
  runner = new Thread( this, ((EventOutSFString)WSC.getEventOut("tag")).getValue() );
  runner.start();
 }

  public TimerEvent(){
  }


 public void eventOutChanged(VrmlEvent vrmlevent ){
	  int b = 0;
	  BaseField bf = vrmlevent.getSource();
    try {
	    if( bf.getType()!= BaseField.SFBool ) {
		System.err.println("Got wrong value from ");
		return;
	    }
     if(sfActiveOut.getValue()==true){
	        bSleep = false;
    if(runner!=null){ runner.interrupt(); } 
      }
     else{
	  bSleep = true;
       }
	    }
	catch(Exception e) {
	    System.err.println("TimerEvent eventOutChanged "+"\n\t" +e);
	}
    }

 
 /**
     *  New 11-12-05 When runner sleeps one hour unless interrupted by EventOut Observer 
     * then it sleeps at designated lSleep intervals pulsing an event true to the world.
     */

 public void run(){
  while( Thread.currentThread() == runner ) {
try {
  if(bSleep){
 Thread.sleep(600000);
   }
  else {
  sfActiveIn.setValue(true);
 Thread.sleep(lSleep);
  }
 }
catch(NullPointerException npe){ 
 System.out.println( "TimerEvent: " + npe );
 }
catch(InvalidEventInException eie){ 
 System.out.println( "TimerEvent: " + eie ); 
 }
catch(InterruptedException e){}
 }
}


  public void delete(){
  if(sfActiveOut!=null){ sfActiveOut.removeVrmlEventListener(this);}
   runner = null; sfActiveOut = null; sfActiveIn = null;
   } 
}