/**
  RoomManager.java
  
  Copyright (C) 1999 Geometrek	and 2009 by R. Russell Kinter
 
 *  This file is part of DeepMatrixIP9.
 *
 *  DeepMatrixIP9 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrix IP9 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.server;

import matrix.TriggerEnvelope;
import matrix.ObjectEnvelope;
import matrix.RealTimeMessage;
import java.util.*;

/**
 * Manages a set of Rooms for the same world. This makes limited numbers of users and
 * several automatically managed copies possible.
 *
 * @version 1.1
 */
public class RoomManager {

/**
 * the list of rooms managed
 */
  private Vector rooms;
/**
 * name and url of the world
 */
  protected String name, url, ownerAddress, sRoompass;
  
  protected boolean bRFM2 = true;

  protected boolean bPublic = true;

  protected String owner = "public";
/**
 * maximum number of users per room representing this world
 */
  private int maxUsers;
/**
 * if true multible copies of this world will be established, if there are too many users
 */
  private boolean multible;
/**
 * reference to MatrixD class
 */
  private MatrixD parent;

  public RoomManager( MatrixD parent, String name, String url, int maxUsers, boolean keep ) {
    rooms = new Vector();
    this.name = name;
    this.url = url;
//  For Controlled Events and SharedObjects one room only will be used for the time being.
	this.maxUsers = 0;
//  this.maxUsers = maxUsers;
     bPublic = true;
	 bRFM2 = keep;
 //  this.multible = multible;
    this.parent = parent;
    rooms.addElement(new Room( parent, name, url, "public", "public", rooms.size(), keep ));
 //   parent.log("RoomManager for "+name+", "+url+" created, 1 room available",3);
  }

 public RoomManager( MatrixD parent, String name, String url, String owner, String addy, String rp, int maxUsers, boolean keep ) {
    rooms = new Vector();
    this.name = name;
    this.url = url;
    this.owner = owner;
    this.sRoompass = rp;
    ownerAddress = addy;
//  For Controlled Events and SharedObjects one room only will be used for the time being.
	this.maxUsers = 0;
//  this.maxUsers = maxUsers;
    this.multible = multible;
    this.parent = parent;
	bRFM2 = keep;
    rooms.addElement(new Room( parent, name, url, owner, rp, rooms.size(), keep ));
 //   parent.log("RoomManager for "+name+", "+url+" created, 1 room available",3);
  }

/**
 * adds a user to the Room managed by this RoomManager. This method creates a new instance of
 * the Room, if necessary and allowed.
 */
  public synchronized Room addUser( ServerThread user ) {
    Enumeration list = rooms.elements();
    boolean found = false;
    Room help;
    if( maxUsers == 0 ) {
      help = (Room) rooms.firstElement();
 // if(help!=null){ 
 // help.addUser( user );	
 //  }
    return help;
    }
//    help = (Room) rooms.firstElement();
//    while( list.hasMoreElements()) {
//      help = (Room)list.nextElement();
//      if( help.countUsers() < maxUsers ) {
//        found = true;
//        break;
//      }
//    }
//    if( found == true ) {
//      help.addUser( user );
//      return help;
//    }
//    else {
//      if( multible == true ) {
//        help = new Room( parent, name, url, "public", "public", rooms.size(), false );
//        rooms.addElement( help );
//        help.addUser( user );
//        parent.log(" New room "+name+" created, now "+rooms.size()+" available",3);
//        return help;
//      }
//    }
    return null;
  }

// Only room to close at the moment
  public void removeFromMatrix(){
  setApproval( false );
  ((Room)rooms.firstElement()).removeFromMatrix();
  }


   public void setApproval( boolean tf ){
	   bRFM2 = tf;
	 ((Room)rooms.firstElement()).bRFM = tf;
   }
 
   public void close(){
  ((Room)rooms.firstElement()).close();
   }
  
  public int getEventsSize(){
  int iZ = ((Room)rooms.firstElement()).getEventsSize();
  return iZ;
  }  

  public boolean getPrivacy(){
  boolean tf = ((Room)rooms.firstElement()).bPublic;
  return tf;
  }

  public void printUpdate( TriggerEnvelope tm ){
   Room rm = (Room)rooms.firstElement();
   rm.saveObjectInfo(tm);
  } 

  public void clearEvents( RealTimeMessage rtm ){
   Room rm = (Room)rooms.firstElement();
   rm.handleStorage( rtm );
  }  

   public void closeWrite(){ 
  ((Room)rooms.firstElement()).finishWrite();
 }
  
  public int getUsers(){
  int num = ((Room)rooms.firstElement()).countAllUsers();
  return num;
  }
  
    public boolean objectInfoNameExists( String nm ){
  boolean tf =  ((Room)rooms.firstElement()).objectInfoNameExists( nm );
  return tf;
  }

   public void setObjectEnvelope( ObjectEnvelope oe ){
	((Room)rooms.firstElement()).saveObjectInfo( oe );
  }
 

/**
 * removes a User from the Room. If a Room is empty, it is removed.
 */
  public synchronized void removeUser( ServerThread user, Room room ) throws InterruptedException {
    room.removeUser( user );
    if( rooms.contains( room )) {
      if( room.countUsers() == 0 && rooms.size() > 3 ) {
        room.close();
        rooms.removeElement( room );
        parent.log("Room "+name+" destroyed, now "+rooms.size()+" available",1);
      }
    }
  }
}
