/**
 ServerThread.java

  Copyright (C) 1999 Geometrek	and 2009 by R. Russell Kinter
 
 *  This file is part of DeepMatrixIP9.
 *
 *  DeepMatrixIP9 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrix IP9 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package matrix.server;

import java.io.*;
import java.net.*;
import java.util.*;
import matrix.*;

/**
 * manages a connection to a certain user. There is one for each user who
 * connects with the Matrix server. This class also maintains the objects
 * created by a user.
 *
 * @version 1.1
 */
public class ServerThread implements Runnable, RemoveUser {
	
	protected MatrixD daemon;
	protected Room room;
	protected Thread runner;
	protected Hashtable objects;

 /** 
   * New 11-12-05 oldRoom Keep track of where user HAD BEEN. In case room change throws error user stays where he is at.
   */

    //private UserObjData UOD = null;
    protected String oldRoom;
    private int  iCMarg	= 0;
    private int  iCMnm = 0;
    private long lTime1 = 0;
    private long lTime2 = 0;
    private long lTimeOut = 7000;
    private   int iCt = 0;
    private  boolean bNetCt = false;
    private   boolean bCMLength = false;
    private   boolean bBot = false; 
    private   boolean bAdminBot = false;
	protected String name;
    protected String passWord = "";
	protected String hostName = "";
	protected boolean bTrans = false;
	protected int clientID;
    //private int iRoomCt = 0;
    private  boolean  bInit = false;
    protected String avurl = "nothin";
    protected int iTransCt = 0;
    private   String apw = "none";
    private   boolean bAdmin = false;
    protected boolean close = false;
    private   boolean tf = false;
	protected String startRoom;
	protected Connection connection;
	protected Hashtable objects2;


	

	/**
	 * creates a new ServerThread to manage the connection with one
	 * client. It creates the necessary Thread object and starts the
	 * thread.  
  */

	public ServerThread(MatrixD daemon, String startRoom, Connection connection) {
		this.connection = connection;	
		objects = new Hashtable();
		this.daemon = daemon; 
		this.startRoom = startRoom;
		runner = new Thread( this );
		runner.start();
	}
	
 public void exchange( ObjectInputStream in, ObjectOutputStream out ) throws Exception {
   connection.exchange( in, out );
 }


	/**
	 * returns the clients name
	 */
	public String getName() {
		return name;
	}

	/** 
   * New 11-12-05 setRoomA is first call for a client's room change.
   * It simply makes them leave the room, but doesn't remove them from the server.
   * A state of stasis, so client can purge memory without having new chat or vrml thrown
   * at it.
   */

 private void setRoomA() throws Exception {
 		oldRoom = room.getName();
			daemon.leaveRoom( this, room ); 
 }

 	/** 
   * New 11-12-05 setRoomB is second call for a client's room change. It really is basically
   * the original old method delayed 4 seconds by setRoomA.  
   */

 private void setRoomB( ControlMessage CM ) throws Exception, IOException {
  try {
					room = daemon.enterRoom( this, CM.name );
					}
					catch(Exception e) {
                     try {
                      String sER = e.getMessage();
					 System.out.println( sER );
                     sendMessage2( new ControlMessage( ControlMessage.ERROR, "MatrixD", sER ));
                        }
                    catch(InterruptedException f){}
                    catch(IOException h) { daemon.log(" " + name +" setRoomB IOException with ID "+clientID + " " + h,1);		}					
						daemon.log("Error changing to "+CM.name+" : "+e.getMessage(),1);
					}
                    if(room==null){
                    String sSR = daemon.startRoom;
                    if(oldRoom!=null&&!oldRoom.equals(CM.name)){
                    sSR = oldRoom;
                    }
					room = daemon.enterRoom( this, sSR );
                    }
                    else {
					sendMessage2( new ControlMessage( ControlMessage.ROOM, room.getName(), room.getWrl() ) );
					room.update( this );
					String[] userList = room.Whois();
					room.postToAll2( new WhoWhatWhere( WhoWhatWhere.WHO, "", userList ) );
					String[] objList = room.Objis();
					sendMessage2( new WhoWhatWhere( WhoWhatWhere.WHAT, "", objList ) );
					String [] st = new String [1];
                    st[0] = (room.getName() + " (" + room.countAllUsers() + ")");
					daemon.postToAllUsersBut2( this, new WhoWhatWhere( WhoWhatWhere.WHRUPD, room.getName(), st ) );
					String[] roomList = daemon.Roomis();
					sendMessage2( new WhoWhatWhere( WhoWhatWhere.WHERE, "", roomList ) );
     daemon.log( "changes to room "+room.getName(), 2 );
                    }
 }

	/**
	 * returns the current Room of the client
	 */
	public Room getRoom() {
		return room;
	}
	
	/**
	 * handles the login handshake. adds the client to the desired or default Room. Returns true if
	 * successful, otherwise false.
	 */
	protected boolean login() {
        int iCheck = 0;
		ControlMessage msg = new ControlMessage();
		try {
			msg.command = ControlMessage.MSG;
			msg.name = "MatrixD";
			msg.arg = "Hello please login to Deep Matrix !";
			sendMessage2(msg);	
			msg.name = "";
			msg.arg = "";
			msg.command = ControlMessage.LOGIN;
			sendMessage2(msg);
			msg =(ControlMessage) connection.getMessage();
			if( msg.command == ControlMessage.LOGIN ) {
            iCheck = daemon.Check( this, msg.name, msg.arg, connection.getInetAddress().getHostName() );
			if( iCheck==1 ) {
                    bTrans = true;
					name = msg.name;
                    passWord = msg.arg;
					bBot = daemon.botPassWord( passWord );
                    bAdminBot = daemon.adminBotPassWord( passWord );
					runner.setName( name );
					clientID = daemon.addThread( this, true );	
					msg.command = ControlMessage.MSG;
					msg.name = "MatrixD";
					msg.arg = ""+clientID;
					sendMessage2(msg);	
                    msg =(ControlMessage) connection.getMessage();
					if( msg.command == ControlMessage.ROOM ) {
						startRoom = msg.name;
     }
    if(msg.arg.equals("BOT")){
  boolean bRoom = false;
  for( int iRCt = 0; iRCt<2; iRCt += 1 ){
	if(iRCt==0){ 
     getRoom( startRoom ); 
     }		
    if(iRCt==1){ 
	sendMessage2( new ControlMessage( ControlMessage.ERROR, "MatrixD", startRoom + " is not available! Trying to return to previous room!" )); 
	 String sSR = daemon.startRoom;
     if(oldRoom!=null&&!oldRoom.equals(startRoom)){
     sSR = oldRoom;
      }
     getRoom( sSR ); 
     }
    if(room!=null){ 
    iRCt = 2; 
  bRoom = true;
    break;
      } 
    }       
  //System.out.println("ServerThread: ControlMessage.ROOM: " + room.getName() + " " + room.getWrl() );
 if(bRoom){
 sendMessage2( new ControlMessage( ControlMessage.ROOM, room.getName(), room.getWrl()));
   //  System.out.println("BOT");
					daemon.log("login from "+connection.getInetAddress().getHostName()+" with ID "+clientID,1);
					daemon.log("start in "+room.getName(),2);
   return true;
      }
   else {
	return false;
    }
   }
      else {
      try {
						sendMessage2( new ControlMessage( ControlMessage.ROOM, startRoom, "CB" ));
      daemon.log("login from "+connection.getInetAddress().getHostName()+" with ID "+clientID,1);
					}
     catch(InterruptedException f){	 
              daemon.log(" " + name +" login failure with ID "+clientID + " " + f,1);
			 return false;}
	catch(IOException e) {
            			 daemon.log(" " + name +" login failure with ID "+clientID + " " + e,1);
                        try {
						sendMessage2( new ControlMessage( ControlMessage.ERROR, "MatrixD", e.getMessage()));
						sendMessage2( new ControlMessage( ControlMessage.LOGOUT, "", "" ));
						  }
                        catch(InterruptedException g){	
                      daemon.log(" " + name +" login failure with ID "+clientID + " " + g,1);
						  return false;}
					    catch(IOException h) {			 
						 daemon.log(" " + name +" login failure with ID "+clientID + " " + h,1);
						return false;
                      }
                      return false;
					}
	  catch(Exception g) {
			   daemon.log(" " + name +" login failure with ID "+clientID + " " + g,1);
                        try {
						sendMessage2( new ControlMessage( ControlMessage.ERROR, "MatrixD", g.getMessage()));
						sendMessage2( new ControlMessage( ControlMessage.LOGOUT, "", "" ));
						  }
                        catch(InterruptedException j){	 
                        daemon.log(" " + name +" login failure with ID "+clientID + " " + j,1);
						  return false;}
					    catch(IOException h) {			  
						 daemon.log(" " + name +" login failure with ID "+clientID + " " + h,1);
						  return false;
                          }
           return false;
						}
      //System.out.println("NotBot");
      return true;
      }
				}
        else {
        if( iCheck==2 ){
			clientID = daemon.addThread( this, false );
		  	msg.command = ControlMessage.ERROR;
			msg.arg = "User name is incorrect! Login failed!";
			sendMessage2(msg);
           bTrans = true;
           return false;
         } 
        else 
		if( iCheck==3 ){
		    clientID = daemon.addThread( this, false );
		  	msg.command = ControlMessage.ERROR;
			msg.arg = "Password is incorrect! Login failed!";
			sendMessage2(msg);
            bTrans = true;
            return false;
         } 
        else 
	   if( iCheck==4 ){
		  	msg.command = ControlMessage.ERROR;
			msg.arg = "User has been banned! Login failed!";
			sendMessage2(msg);
			return false;
         } 
        else
       if( iCheck==5 ){
		  	msg.command = ControlMessage.ERROR;
			msg.arg = "User is already logged in! Login failed!";
			sendMessage2(msg);
		   return false;
         } 
	    if( iCheck==6 ){
		  	msg.command = ControlMessage.ERROR;
			msg.arg = "Too many users from one address! Login failed!";
			sendMessage2(msg);
		   return false;
           } 
		 daemon.log("Login failure: "+msg.arg,1);
         }
			}
		//	msg.command = ControlMessage.ERROR;
		//	msg.arg = "Login failed!";
		//	sendMessage2(msg);
		}
  catch(InterruptedException f){ daemon.log("Login "+f,1);return false;}
  catch(NullPointerException n){ daemon.log("Login "+n,1); return false; }
  catch(IOException g){ daemon.log("Login "+g,1); return false; }
		catch(Exception e) {
			daemon.log("Login Error "+e,1);
		  return false;
		}
		return false;
	}
	
	/**
	 * handles the connection. First it calls login, then continues with a loop, polling the
	 * connection for new messages and handling the client requests.
	 */
	public void run() {
		Message msg = null;
		if(login()==false) {
	 // System.out.println("ST login()==false");
   try {
			connection.close();
   }
   catch(InterruptedException f){}
			daemon.log("Login failed",1);
     if(bTrans){
			try {
        daemon.userTransition( new Integer(clientID) );
         }
        catch( Exception e ){
         daemon.log(" " + clientID + " " + name + " userTransition exception: " + e,1); 
          }
        }
			daemon.Threads.remove(new Integer(clientID));

			return;
		}
	String[] roomList = daemon.Roomis();
    try {
	sendMessage2( new WhoWhatWhere( WhoWhatWhere.WHERE, "", roomList ) );
    sendMessage2( new ControlMessage( ControlMessage.HRTBT, "init", "hrbt" ));
     }
	  catch( InterruptedException f ){ }
	catch(IOException e){ goodBye(); daemon.log(" ServerThread run: " + name + " " + e,1);}
   //if(1<daemon.passLevel&&!bTrans){ UOD = new UserObjData( daemon.getAbsolutePath(), name, this );   }
	hostName = connection.getInetAddress().getHostName();
		try {
			while( Thread.currentThread() == runner ) {
				msg = connection.getMessage();
       if(!bAdminBot&&msg!=null){
        lTime2 = System.currentTimeMillis();
           iCt +=1;
	   if(60<iCt){
       if(lTime2<(lTime1+lTimeOut)){
        	bNetCt = true;
     //   System.out.println("ServerThread Message count issue issue");
            runner = null; }
          else { 
          lTime1 = lTime2; iCt = 0; 
               }
             }
            }
				switch( msg.getType()) {
				case ControlMessage.TYPE :          // so we got our good old control message
                 ControlMessage cm = (ControlMessage)msg;
					if( 175<cm.arg.length()||25<cm.name.length() ){ iCMarg = cm.arg.length(); iCMnm = cm.name.length(); bCMLength = true; runner = null; }
					if( cm.command == ControlMessage.ERROR ) {
						daemon.log(cm.toString(),1);
						sendMessage2(cm);
						continue;
					}
					else 
               if(cm.command!=ControlMessage.HRTBT){ daemon.log(cm.toString(),3); }
				switch( cm.command ) {    // dealing with the different commands
				case ControlMessage.OBJADD :
 					if( objects.containsKey( cm.name )){
						cm.command = ControlMessage.ERROR;
						cm.arg = "Object "+cm.name+" exists allready !";
						cm.name = "MatrixD";
						sendMessage2( cm );
						break;
					}
					if( room.addObject( this, cm.name, cm.arg ) == false ){
						cm.command = ControlMessage.ERROR;
						cm.arg = "Object "+cm.name+" exists allready in room "+room.getName() + " !";
						cm.name = "MatrixD";
						sendMessage2( cm );
						break;
					}
					if(cm.name.startsWith("_"+this.name)){ 
                    avurl = cm.arg;
                    objects.put( cm.name, cm.arg ); 
                    }
					daemon.log("adds "+cm.name+"("+cm.arg+")",2);
					break;
				case ControlMessage.OBJDEL :
					room.deleteObject( this, cm.name );
					objects.remove( cm.name );
					daemon.log("deletes "+cm.name,2);
					break;
				case ControlMessage.OBJREQ :
					//room.requestObject( name, cm.name, cm.arg );
					break;
				case ControlMessage.UPDATE :
                    Enumeration update = null; 
					if( cm.name.equals( "world" )){
					  update = room.updateRoutesEnum();
					  ControlEvent msg2 = new ControlEvent();
		             msg2.command = ControlEvent.CTRLEV;
		           while( update.hasMoreElements()) {
			       String [] help = (String [])update.nextElement();
                 if(help[0].indexOf("#")==-1){
                 msg2.name = help[0];
			     msg2.arg = new String []  {help[1]};
				  sendMessage2( msg2 );
                    }
                   }
                      update = room.updateEventsEnum();
            		 while( update.hasMoreElements()){
                   RealTimeMessage rtm = (RealTimeMessage)update.nextElement();
                   if(rtm.tag.indexOf("#")==-1){
                  sendMessage( rtm );
                        }
                      }
                     } 
                   else {   
				room.updateObject( cm.name );
					}
					break;
                case ControlMessage.OBJUPD :
                if(room.SharedObjectExists( cm.name )){
				Message [] msgs = room.updateSharedObjectMsgs( cm.name );
				if(msgs.length==2){
                sendMessage( (RealTimeMessage)msgs[0] );
                sendMessage( (RealTimeMessage)msgs[1] );
                  }
                if(msgs.length==1){
                sendMessage3( (ControlMessage)msgs[0] );
                }
			Enumeration eventobjs = room.updateEventsEnum();
       while( eventobjs.hasMoreElements()){
       RealTimeMessage rtm = (RealTimeMessage)eventobjs.nextElement();
       if(rtm.tag.startsWith(cm.name+"#")){
		 sendMessage( rtm );
          }
        }
        eventobjs = room.updateRoutesEnum();
		ControlEvent msg2 = new ControlEvent();
		msg2.command = ControlEvent.CTRLEV;
		while( eventobjs.hasMoreElements()) {
			String [] help = (String [])eventobjs.nextElement();
        if(help[0].startsWith(cm.name + "#" )){
        // System.out.println("Room updateSharedObject help[0]: " + help[0] + " help[1]: " + help[1] );
            msg2.name = help[0];
			msg2.arg = new String []  {help[1]};
			sendMessage2( msg2 );
          }
         }
        }
        else {
    	 sendMessage3( new ControlMessage( ControlMessage.ERROR, "MatrixD", "Object " + cm.name + " does not exist in this room!") );
         }
                    break;
				case ControlMessage.IGG :
               // daemon.handleAnnoyer( cm.name, name, cm.arg );
                break;
				case ControlMessage.MSG :
					String help = cm.name;
					cm.name = name;
					if(help.equals("ALL"))
						room.postToAllBut3( this, cm );
					else
						room.postTo( help, cm );
					break;
				case ControlMessage.WHOIS :
                    WhoWhatWhere www = (WhoWhatWhere)msg;
					String[] list = room.Whois();
					www.command = WhoWhatWhere.WHO;
					www.name = "";     
					www.wwwe = list;
					sendMessage2(www);
				 case ControlMessage.OBJIS :
					WhoWhatWhere www2 = (WhoWhatWhere)msg;
					String[] list2 = room.Objis();
					www2.command = WhoWhatWhere.WHAT;
					www2.name = "";
					www2.wwwe = list2;
					sendMessage2(www2);
					break;
				  case ControlMessage.ROOMIS :
					WhoWhatWhere www3 = (WhoWhatWhere)msg;
					String[] list3 = daemon.Roomis();
					www3.command = WhoWhatWhere.WHERE;
					www3.name = "";
					www3.wwwe = list3;
					sendMessage2(www3);
					break;
             case ControlMessage.HRTBT :
                 if(!bInit){ 
					Enumeration names = null;
                    ControlMessage hbt = (ControlMessage)msg;
                    room.addUser( this ); 
				    Hashtable ht = room.getObjects();
                    hbt.command = ControlMessage.OBJADD;   
					names = ht.keys();
		           while( names.hasMoreElements()) {
			       hbt.name = (String)names.nextElement();
			       hbt.arg = (String) ht.get( hbt.name );
				    sendMessage2( hbt );
                    }
                    names = room.addLoadedObjectsEnum();
					while( names.hasMoreElements()){
                       ObjInfo OI3 = ((ObjInfo)names.nextElement());
	              if(0<OI3.objState){
                  ControlMessage cmo = new ControlMessage( ControlMessage.OBJADD, OI3.objName, OI3.objUrl );
	                sendMessage3(  cmo );
                      }
                    }
					hbt.command = ControlMessage.MSG;
					hbt.name = name;
					hbt.arg = " logged in";
					room.postToAllBut2( this, hbt );
					String[] userList = room.Whois();
                    Message wwww = new WhoWhatWhere( WhoWhatWhere.WHO, "", userList );
					room.postToAllBut2( this, wwww );
                    sendMessage2( wwww );
					String[] objList = room.Objis();
					sendMessage2( new WhoWhatWhere( WhoWhatWhere.WHAT, "", objList ) );
					String [] st = new String [1];
                    st[0] = (room.getName() + " (" + room.countAllUsers() + ")");
                    wwww = new WhoWhatWhere( WhoWhatWhere.WHRUPD, room.getName(), st );
					daemon.postToAllUsersBut2( this, wwww );
                    sendMessage2( wwww );
                  bInit = true; }
						break;
				case ControlMessage.ROOM :
					if(cm.arg.equals("A")){
				 setRoomA();
     sendMessage2( new ControlMessage( ControlMessage.ROOM, cm.name,"CB"));
      } 
     if(cm.arg.equals("B")){ 
      ControlMessage msg2 = new ControlMessage();
      setRoomB( cm );
	  daemon.log("start in "+room.getName(),2);
     }
     if(cm.arg.equals("C")){
      setRoomA();
	     setRoomB( cm );
          }
					break;
				case ControlMessage.LOGOUT :
                oldRoom = startRoom;
			    if(cm.arg.equals("true")){ bTrans = true; }
                 else { bTrans = false; }
				runner = null;
					break;
                case ControlMessage.NEWROOM	:
                daemon.addNewRoom( name, cm.name, cm.arg, connection.getInetAddress().getHostName() );
                    break;
                case ControlMessage.ADMIN:
               AdminControl ac;
               if(daemon.adminID==-1){
                if(daemon.getAdminPassword().equals(cm.arg)){
				 apw = cm.arg;
                 ac = new AdminControl( AdminControl.HELLO, cm.name, Integer.toString(clientID) );
                 bAdmin = true;
				 daemon.adminAction( ac );
                }
                else {
            	 ac = new AdminControl( AdminControl.REJCT, cm.name , "Admin password incorrect!" );
				try {
				sendMessage2( ac );
			        }
            catch( InterruptedException f ){ }
			catch(IOException e){}
                 }
                }
                else {
				 ac = new AdminControl( AdminControl.REJCT, cm.name , "Admin is being used!" );
				try {
				sendMessage2( ac );
			        }
            catch( InterruptedException f ){ }
			catch(IOException e){}
                  }
                   break;
				default :
					daemon.log("unknown control message "+cm,1);
				}
				break;
				case RealTimeMessage.TYPE :         // its a real time message
               RealTimeMessage rtm = (RealTimeMessage)msg;
              if(rtm.echo == true ){
						room.postToAll( rtm );
                    }
					else {
						room.postToAllBut( this, rtm );
                    }
               switch( rtm.persist ) { 
                case RealTimeMessage.NONE :
                break;
				case RealTimeMessage.POP :
               room.handleStorage( rtm );
                break;
			   	case RealTimeMessage.FOR :
               room.handleStorage( rtm );
			   case RealTimeMessage.CLR :
               room.handleStorage( rtm );
                break;
				 default :
					daemon.log("unknown RealTimeMessage persist "+rtm,1);
                break;
                }
                break;
				case RealTimeSave.TYPE :
			    RealTimeSave rts = (RealTimeSave)msg;
    			ObjectEnvelope OE = new ObjectEnvelope( rts.tag, name, rts.data, rts.state ); 
                   room.saveObjectInfo( OE );
                break;
        			case AdminControl.TYPE : 
                AdminControl act = (AdminControl)msg;
                if(daemon.getAdminPassword().equals(apw)){
				  daemon.adminAction( act );
                 }
                 else {
				 daemon.log(" phony admin attempt: "+act.toString(),1);
                 runner = null; 
                 }
                break;
			    case ControlEvent.TYPE :
                room.sendControlEvent( msg );
                break;
			    case RoomControl.TYPE : 
				room.sendControlEvent( msg );
                RoomControl rc = (RoomControl)msg;
                break;
                case ObjectEnvelope.TYPE :
                break;
				default :
					daemon.log("unknown message type "+msg,1);
                break;
				 }
			   
		 }
  }
  //SocketException
  catch(InterruptedException f){ goodBye(); daemon.log(name +": "+f,1); }
  catch(NullPointerException n){ goodBye(); daemon.log(name +": "+n,1);}
  catch(EOFException d) { goodBye(); daemon.log(name +": "+d,1); }
  catch(SocketTimeoutException g){ goodBye(); daemon.log(name +": "+g,1);  }
  catch(IOException h){ goodBye(); daemon.log(name +": "+h,1); }
		catch(Exception e) {
   goodBye();
			daemon.log("Error "+e,1);
		}
		finally {
		//	if(UOD!=null){ UOD.stop(); }
            if(bAdmin){	
            //System.out.println("ST finally bAdmin: " + bAdmin );
            daemon.adminAction( new AdminControl( AdminControl.DONE, "", "finis" ) );
            apw = "none"; bAdmin = false; 
            }
			// clean up
			ControlMessage cm = new ControlMessage();
			cm.command = ControlMessage.OBJDEL;
			cm.arg = "";
			if( room != null ) {
				Enumeration list = objects.keys();
				while( list.hasMoreElements()){
					cm.name = (String) list.nextElement();
    try {
					room.deleteObject( this, cm.name );
      }
    catch(InterruptedException f){}
					daemon.log("forgot "+cm.name+" !",1);
				}
				cm.command = ControlMessage.MSG;
				cm.name = name;
				cm.arg = "logged out";
				room.postToAllBut2( this, cm );
				try {
					daemon.leaveRoom( this, room );
				}
				catch(Exception e){}
			}
			cm.command = ControlMessage.LOGOUT;
			cm.name = name;
			cm.arg = "";
			try {
				sendMessage2( cm );
			}
   catch( InterruptedException f ){ }
			catch(IOException e){}
   try {
			connection.close();
   }
   catch( InterruptedException e ){ }
         if(bCMLength){
		 daemon.log(" " + clientID + " " + name + " ControlMessage length excessive arg length: " + iCMarg + " name length: " + iCMnm + " ",1);
         }
         if(bNetCt){
    	 daemon.log(" " + clientID + " " + name + " Message overload iCt: " + iCt + " lTime1: " + lTime1 + " lTime2: " + lTime2  ,1);
         }
			 if(bTrans){
         try {
        daemon.userTransition( new Integer(clientID) );
         }
        catch( Exception e ){
         daemon.log(" " + clientID + " " + name + " userTransition exception: " + e,1); 
          }
    	 daemon.log(" " + clientID + " " + name + " changing rooms. ",1);
        } 
        else {
         try {
           daemon.setIPCount( hostName );
         }
	  catch( Exception e ){
         daemon.log(" " + clientID + " " + name + " IPCount exception: " + e,1); 
          }
		 daemon.log("logged out",1);
        }
			daemon.Threads.remove(new Integer(clientID));
		}
	}


	/**
	 * tells the client to send new updates for object with name name.
	 * Tests whether this client holds this object and sends an update
	 * messge if so.
	 *
	 * @param name the name of the object
	 */    
	public void updateObject( String name ){
		if( connection.isAlive() &&  objects.containsKey( name )) {
			try {
				sendMessage2( new ControlMessage( ControlMessage.UPDATE, name, "" ));
			}
   catch( InterruptedException f ){ }
			catch( IOException e ){
				daemon.log("Error updating object "+name+" "+e,1);
			}
		}    
	}
	
	public void sendMessage( Message msg ) throws IOException {
		connection.sendMessage( msg );
	}

 public void sendMessage2( Message msg ) throws IOException, InterruptedException {
		connection.sendMessage2( msg );
	}

 public void sendMessage3( Message msg ) throws IOException {
		connection.sendMessage3( msg );
	}
 
 public boolean getbBot(){
 return bBot;
 }

 private void getRoom( String sRoom ){
  try {
    room = daemon.enterRoom( this, sRoom );
         }		
    catch(Exception e) {
	daemon.log("ST login arg equals BOT exception thrown:" + e,1);
        try {
		String sER = e.getMessage();
		sendMessage2( new ControlMessage( ControlMessage.ERROR, "MatrixD", sER ));
	    System.out.println( sER );
		sendMessage2( new ControlMessage( ControlMessage.LOGOUT, "", "" ));
                        }
		catch(InterruptedException f){	 
        daemon.log(" " + name +" login failure with ID "+clientID + " " + f,1);
          }
		     catch(IOException h) { 
            daemon.log(" " + name +" initial Exception error LOGOUT failure with ID "+clientID + " " + h,1); }					
			daemon.log("start room "+startRoom+" not available, login break off",1);
					}
            }		

  public boolean getbAdminBot(){
 return bAdminBot;
 }

 public void removeUser(){
 }

	
 public	void close() {
		goodBye();
 }

  synchronized void goodBye(){
  runner = null;
  }
  

public void removeObject( String name ){ 
if(objects.containsKey( name )){
objects.remove( name );
  }
 }

}
