/* copyright (c) 1997 stephen f. white
 *  This file is part of DeepMatrix V1.
 *
 *  DeepMatrix V1 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrix V1 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.vrml;

import java.io.*;


/**
 * Abstract prototype for all VRML fields. Originally by Stephen F. White.
 * Additional features by Gerhard Reitmayr<BR>
 * Field classes are independent of vrml.* again. Any relation to that should be handled by the code
 * itself.
 */
public abstract class Field
{
  public static final byte NOTHING = 0;

  public static final byte SFBOOL = 1;
  public static final byte SFCOLOR = 2;
  public static final byte SFFLOAT = 3;
  public static final byte SFIMAGE = 4;
  public static final byte SFINT32 = 5;
  public static final byte SFNODE = 6;
  public static final byte SFROTATION = 7;
  public static final byte SFSTRING = 8;
  public static final byte SFTIME = 9;
  public static final byte SFVEC2F = 10;
  public static final byte SFVEC3F = 11;

  public static final byte MFCOLOR = 12;
  public static final byte MFFLOAT = 13;
  public static final byte MFINT32 = 14;
  public static final byte MFNODE = 15;
  public static final byte MFROTATION = 16;
  public static final byte MFSTRING = 17;
  public static final byte MFVEC2F = 18;
  public static final byte MFVEC3F = 19;

  public static final byte FIELDARRAY = 20;

  public byte getType() { return NOTHING; }

  public abstract void write( DataOutputStream out ) throws IOException;
  public abstract void set( Field data ) throws MatrixInvalidFieldException;


}
