/* copyright (c) 1997 stephen f. white
 *  This file is part of DeepMatrix V1.
 *
 *  DeepMatrix V1 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrix V1 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.vrml;

import java.io.*;
import java.util.*;

/**
 * A class for holding an array of fields.
 */
public class FieldArray extends Field {
  Vector data;

  public FieldArray(){
    data = new Vector();
  }

  public FieldArray( Field[] data ){
    this.data = new Vector(data.length);
    for( int i = 0; i < data.length; i++ ){
      this.data.addElement(data[i]);
    }
  }

  public FieldArray(InputStream in) throws IOException {
    FieldInputStream fin = new FieldInputStream( in );
    int size = fin.readInt();
    data = new Vector(size);
    for( int i = 0; i < size; i++ ){
      data.addElement( fin.readField());
    }
  }

  public void write(DataOutputStream out) throws IOException {
    FieldOutputStream fout = new FieldOutputStream( out );
    fout.writeInt( data.size());
    for( int i = 0; i < data.size(); i++ ){
      fout.writeField((Field)data.elementAt(i));
    }
  }

  public byte getType() { return FIELDARRAY; }

  public void set( Field data ) throws MatrixInvalidFieldException {
    if( data.getType() != FIELDARRAY )
      throw new MatrixInvalidFieldException("Data not FieldArray field");
    this.data =(Vector)((FieldArray)data).data.clone();
  }

  public void addField( Field what ){
    data.addElement( what );
  }

  public int size(){
    return data.size();
  }

  public Field fieldAt( int i ){
    return (Field)data.elementAt( i );
  }

  public Field[] copyInto(){
    Field[] fields = new Field[ data.size()];
    data.copyInto( fields );
    return fields;
  }
}
