/* copyright (c) 1997 stephen f. white
 *  This file is part of DeepMatrix V1.
 *
 *  DeepMatrix V1 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrix V1 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.vrml;

import java.io.*;

/**
 * A class for MFColor field
 * <BR>originally by Stephen F. White, additional features by Gerhard Reitmayr
 */
public class MFColor extends Field implements Serializable {
  SFColor[] colors;

  public MFColor( float[][] data ){
    setValue( data );
  }

  public MFColor( MFColor data ) {
    setValue( data.getValue());
  }

  public MFColor(DataInputStream in) throws IOException {
    colors = new SFColor[in.readInt()];
    for (int i = 0; i < colors.length; i++) {
      colors[i] = new SFColor(in);
    }
  }

  public void setValue( float[][] data ){
    colors = new SFColor[data.length];
    for( int i = 0; i < data.length ;i++ ) {
      colors[i] = new SFColor( data[i] );
    }
  }

  public void set1Value( int index, float[] value ){
    colors[index] = new SFColor( value );
  }

  public SFColor[] getColors() {
    return colors;
  }

  public SFColor get1Color( int pos ) {
    return colors[pos];
  }

  public float[][] getValue() {
    float[][] out = new float[colors.length][3];
    for( int i = 0 ; i < colors.length ; i ++ ){
      out[i][0] = colors[i].red;
      out[i][1] = colors[i].green;
      out[i][2] = colors[i].blue;
    }
    return out;
  }

  public float[] get1Value( int pos ){
    return colors[pos].getValue();
  }

  public void write(DataOutputStream out) throws IOException {
    out.writeInt(colors.length);
    for (int i = 0; i < colors.length; i++) {
      colors[i].write(out);
    }
  }

  public byte getType() { return MFCOLOR; }

  public void set( Field data ) throws MatrixInvalidFieldException {
    if( data.getType() != MFCOLOR )
      throw new MatrixInvalidFieldException("Data not MFColor field");
    setValue(((MFColor)data).getValue());
  }
}
