/* copyright (c) 1997 stephen f. white
 *  This file is part of DeepMatrix V1.
 *
 *  DeepMatrix V1 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrix V1 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.vrml;

import java.io.*;

/**
 * A class for MFFloat field
 * <BR>originally by Stephen F. White, additional features by Gerhard Reitmayr
 */
public class MFFloat extends Field implements Serializable {
  public float[] values;

  public MFFloat( float[] val ) {
    setValue( val );
  }

  public MFFloat( MFFloat val ){
    setValue( val.values );
  }

  public MFFloat(DataInputStream in) throws IOException {
    values = new float[in.readInt()];
    for (int i = 0; i < values.length; i++) {
      values[i] = in.readFloat();
    }
  }

  public void write(DataOutputStream out) throws IOException {
    out.writeInt(values.length);
    for (int i = 0; i < values.length; i++) {
      out.writeFloat(values[i]);
    }
  }

  public byte getType() { return MFFLOAT; }

  public void set( Field data ) throws MatrixInvalidFieldException {
    if( data.getType() != MFFLOAT )
      throw new MatrixInvalidFieldException("Data not MFFloat field");
    setValue(((MFFloat)data).values);
  }

  public void setValue( float[] data ){
    values = new float[data.length];
    for( int i = 0; i< values.length; i++ )
      values[i] = data[i];
  }

  public void set1Value( int index, float data ){
    values[index] = data;
  }

  public float[] getValue(){
    return values;
  }

  public float get1Value(int index){
    return values[index];
  }
}

