/* copyright (c) 1997 stephen f. white
 *  This file is part of DeepMatrix V1.
 *
 *  DeepMatrix V1 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrix V1 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.vrml;

import java.io.*;

/**
 * A class for MFVec3f field
 * <BR>originally by Stephen F. White, additional features by Gerhard Reitmayr
 */
public class MFVec3f extends Field implements Serializable {
  public SFVec3f[] values;

  public MFVec3f( float[][] val ){
    setValue( val );
  }

  public MFVec3f( MFVec3f val ){
    setValue( val.getValue());
  }

  public MFVec3f(DataInputStream in) throws IOException {
    values = new SFVec3f[in.readInt()];
    for (int i = 0; i < values.length; i++) {
      values[i] = new SFVec3f(in);
    }
  }

  public void write(DataOutputStream out) throws IOException {
    out.writeInt(values.length);
    for (int i = 0; i < values.length; i++) {
      values[i].write(out);
    }
  }

  public byte getType() { return MFVEC3F; }
  public void set( Field data ) throws MatrixInvalidFieldException {
    if( data.getType() != MFVEC3F )
      throw new MatrixInvalidFieldException("Data not MFVec3f field");
    setValue(((MFVec3f)data).getValue());
  }

  public void setValue( float[][] data ){
    values = new SFVec3f[data.length];
    for( int i = 0; i < data.length ;i++ ) {
      values[i] = new SFVec3f( data[i] );
    }
  }

  public void set1Value( int index, float[] value ){
    values[index] = new SFVec3f( value );
  }

  public SFVec3f[] getVec3f() {
    return values;
  }

  public SFVec3f get1Vec3f( int pos ) {
    return values[pos];
  }

  public float[][] getValue() {
    float[][] out = new float[values.length][3];
    for( int i = 0 ; i < values.length ; i ++ ){
      out[i] = values[i].getValue();
    }
    return out;
  }

  public float[] get1Value( int pos ){
    return values[pos].getValue();
  }
}
