/* copyright (c) 1997 stephen f. white
 *  This file is part of DeepMatrix V1.
 *
 *  DeepMatrix V1 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrix V1 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.vrml;

import java.io.*;

/**
 * A class for SFVec3f field
 * <BR>originally by Stephen F. White, additional features by Gerhard Reitmayr
 */
public class SFVec3f extends Field implements Serializable {
  public float[] values = new float[3];

  public SFVec3f(){}

  public SFVec3f( SFVec3f data ) {
    setValue( data.getValue());
  }

  public SFVec3f(float x, float y, float z){
    values[0] = x;
    values[1] = y;
    values[2] = z;
  }

  public SFVec3f(float[] values){
    setValue( values );
  }

  public SFVec3f(DataInputStream in) throws IOException {
    values[0] = in.readFloat();
    values[1] = in.readFloat();
    values[2] = in.readFloat();
  }

  public void write(DataOutputStream out) throws IOException {
    out.writeFloat(values[0]);
    out.writeFloat(values[1]);
    out.writeFloat(values[2]);
  }

  public byte getType() { return SFVEC3F; }

  public void set( Field data ) throws MatrixInvalidFieldException {
    if( data.getType() != SFVEC3F )
      throw new MatrixInvalidFieldException("Data not SFVec3f field");
    setValue(((SFVec3f)data).getValue());
  }

  public SFVec3f plus(SFVec3f v) {
    return new SFVec3f(values[0] + v.values[0],
           values[1] + v.values[1],
           values[2] + v.values[2]);
  }

  public SFVec3f minus(SFVec3f v) {
    return new SFVec3f(values[0] - v.values[0],
           values[1] - v.values[1],
           values[2] - v.values[2]);
  }

  public void setValue( float[] value ){
    values[0] = value[0];
    values[1] = value[1];
    values[2] = value[2];
  }

  public float[] getValue(){
    return values;
  }

}
