/*
  Bot.java

  Copyright (C) 1998 Gerhard Reitmayr
*/

package tools;

import matrix.Message;
import matrix.deck.*;
import java.io.*;
import java.util.*;

/**
 * Bot Client for the Matrix System. used to test server loads etc.
 * uses special RealTimeConsumers to simulate users, etc. Reads its data from a resource file
 * with the following format :
 * <pre>hostname port name roomname
 * tools.name arg1 arg2 arg3 ...
 * # comments must start with '#' and be on a line by themselves
 * ...
 * </pre>
 * <tt>tools.name</tt> is the name of the class Bot should load. It must implement the BotConsumer
 * interface. The rest of the line are space separated args ( max 10 ) to the BotConsumer, given it via setData.
 *
 * @version 0.5
 * @author Gerhard Reitmayr
 * @see tools.BotConsumer
 */
public class Bot implements NetworkListener {

  // Matrix stuff
  private Backend net;
  private Heart heart;
  private Vector consumers = new Vector();
  private String name = "";
  private String password = "password";

  public static void main( String[] args ) throws Exception {
    Bot me;
 System.err.println(args[0]);
    if( args.length < 1 ) {
      System.err.println("Usage : java tools.Bot resourcefile");
      return;
    }
    try{     
      me = new Bot( args[0] );
      while( new FileInputStream(args[0]).available() == 0){
        try{
            Thread.sleep(1000);
        }
        catch(InterruptedException e){}
      }
    }
    catch(Exception e){
      System.err.println("Error running Bot "+e);
      System.exit(1);
    }
  }

  public Bot( String file ) throws Exception {
    net = new Backend( this, 1 );

    // read resource file
    BufferedReader fin = new BufferedReader(new FileReader(file));

    // must be arguments for connecting
    String line = fin.readLine();
    while( line.startsWith("#") )
      line = fin.readLine();
    StringTokenizer token = new StringTokenizer( line, " " );
    String host = token.nextToken();
    int port = Integer.parseInt( token.nextToken());
    this.name = token.nextToken();
    this.password = token.nextToken();
    String room = token.nextToken();
	room = room = room.replace('_',' ');
    System.out.println("Connecting to "+host+":"+port+" as "+name+" in room "+room+".");

    // read the rest of the lines
    line = fin.readLine();
    Class consumerClass;
    Object consumerObject;
    String classname, args[];
    Vector argvec = new Vector();
    while( line != null ){
      if( line.length() == 0 || line.startsWith("#")){
        line = fin.readLine();
        continue;
      }
      token = new StringTokenizer( line, " " );
      consumerClass = Class.forName( token.nextToken());
      consumerObject = consumerClass.newInstance();
      if( !(consumerObject instanceof tools.BotConsumer )) {
        System.err.println("Class "+consumerClass+" does not implement BotConsumer");
        continue;
      }
      argvec.removeAllElements();
      while( token.hasMoreTokens()){
        argvec.addElement( token.nextToken());
      }
      args = new String[ argvec.size()];
      argvec.copyInto( args );
      consumers.addElement( consumerObject );
      ((BotConsumer)consumerObject).setData( net, args );

      line = fin.readLine();
    }
    fin.close();

    // init network and open connection
    net.connect( host, port, name, password, room, "BOT" );

    // start all BotConsumers
    Enumeration list = consumers.elements();
   System.out.println("try1");
  try {
    while( list.hasMoreElements()){
      ((BotConsumer)list.nextElement()).start();}}
  catch( NoSuchElementException e ){System.err.println("gotcha");} 
  }

  public void handleHeartStart(){
   heart = new Heart( this, 5000 );
  }

  public void handleLOGOUT() {
    System.out.println("LOGOUT received ... shutting down");
    System.exit(0);
  }

  public void handleOBJADD( String name, String url ){
    System.out.println("Avatar "+name+"("+url+") added");
    net.addConsumer( new ObjectListener( name, false));
  }

  public void handleOBJDEL( String name ){
    net.removeConsumer( name );
    System.out.println("Avatar "+name+" removed");
  }

  public void handleMSG( String from, String line ){
    System.out.println(from+" : "+line);
  }

  public void handleWHOIS( String[] users ){
    System.out.print("Userlist : ");
    for( int i = 0; i < users.length; i++)
      System.out.print(users[i]+" ");
    System.out.println("");
  }

  public void handleERROR( String from, String line ){
    System.out.println(from+" : "+line);
  }

  public void handleCTRLEV( String name, String [] target ){
  System.out.println( "Bot CTRLEV: name " +  name );
  }

    public void handleNEWRM( String new_room ){
  }


  public void handleNetFailure( String reason ){
  System.out.println( "Login failure: " + reason );
  }

  public void handleOBJIGN( String name, String tf ){
  }
  
  public void handleROOM_A( String name ) throws InterruptedException {
  }
  
  public void handleROOM_B( String name, String url ) throws InterruptedException {
  }

  public void handleOBJIS( String[] objects ){
  }

   public void handleWWW( Message msg ){
  }

  public void handleADMIN( int arg ){
   }

   public void handlePOLL( float fsto ){
   }

  public void handleNavChange( String [] hnc ){
   }

  public void handleSpeedChange( float ft ){
   }

    public void heartBeat() throws IOException {
  net.sendHeartBeat( name );
     }

  public void close(){
     //System.out.println("Bot close");
    // stop all BotConsumers
    Enumeration list = consumers.elements();
    while( list.hasMoreElements())
      ((BotConsumer)list.nextElement()).stop();
    heart.stop();
    net.disconnect( "false" );
    net.close();
  }
}
